"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

This is a simple class to create and manage a connection to an Egeria backend. It is the Superclass for the
different client capabilities. It also provides the common methods used to make restful self.session to Egeria.

"""
import asyncio
import inspect
import json
import os
import httpx
from httpx import AsyncClient, Response

from pyegeria._exceptions import (
    OMAGCommonErrorCode,
    InvalidParameterException,
    PropertyServerException,
    UserNotAuthorizedException )
from pyegeria._globals import max_paging_size, TEMPLATE_GUIDS, INTEGRATION_GUIDS
from pyegeria._validators import (
    validate_name,
    validate_server_name,
    validate_url,
    validate_user_id,
    is_json
)

...


class Client:
    """
    An abstract class used to establish connectivity for an Egeria Client
    for a particular server, platform and user.

    Attributes
    ----------
        server_name : str (required)
            Name of the OMAG server to use
        platform_url : str (required)
            URL of the server platform to connect to
        user_id : str
            The identity of the user calling the method - this sets a default optionally used by the methods
            when the user doesn't pass the user_id on a method call.
        user_id : str (required)
            The identity used to connect to the server
        user_pwd : str
            The password used to authenticate the server identity

    Methods
    -------
        create_egeria_bearer_token(user_Id: str, password: str = None) -> str
           Create a bearer token using the simple Egeria token service - store the bearer token in the object instance.

        refresh_egeria_bearer_token()-> None
            Refresh the bearer token using the attributes of the object instance.

        set_bearer_token(token: str) -> None
            Set the bearer token attribute in the object instance - used when the token is generated
            by an external service.

        get_token() -> str
            Retrieve the bearer token.

        make_request(request_type: str, endpoint: str, payload: str | dict = None,
                    time_out: int = 30) -> Response
            Make an HTTP Restful request and handle potential errors and exceptions.

    """

    json_header = {"Content-Type": "application/json"}

    def __init__(
            self,
            server_name: str,
            platform_url: str,
            user_id: str = None,
            user_pwd: str = None,
            verify_flag: bool = False,
            api_key: str = None,
            page_size: int = max_paging_size,
            token: str = None,
            token_src: str = None,
            async_mode: bool = False
    ):
        self.server_name = None
        self.platform_url = None
        self.user_id = user_id
        self.user_pwd = user_pwd
        self.ssl_verify = verify_flag
        self.page_size = page_size
        self.token_src = token_src
        self.token = token
        self.sync_mode = async_mode
        self.exc_type = None
        self.exc_value = None
        self.exc_tb = None

        #
        #           I'm commenting this out since you should only have to use tokens if you want - just have to
        #           create or set the token with the appropriate methods as desired.
        # if token is None:
        #     token = os.environ.get("Egeria_Bearer_Token", None)
        #     if token is None: # No token found - so make one
        #         self.create_egeria_bearer_token(self.user_id, self.user_pwd)
        #     else:
        #         self.token = token

        if api_key is None:
            api_key = os.environ.get("API_KEY", None)
        self.api_key = api_key

        self.headers = {
            "Content-Type": "application/json",
        }
        self.text_headers = {
            "Content-Type": "text/plain",
        }
        if self.api_key is not None:
            self.headers["X-Api-Key"] = self.api_key
            self.text_headers["X-Api-Key"] = self.api_key

        if token is not None:
            self.headers["Authorization"] = f"Bearer {token}"
            self.text_headers["Authorization"] = f"Bearer {token}"

        v_url = validate_url(platform_url)

        if v_url:
            self.platform_url = platform_url
            if validate_server_name(server_name):
                self.server_name = server_name
            # if self.sync_mode:
            #     self.session = httpx.Client(verify=self.ssl_verify)
            # else:
            #     self.session = httpx.AsyncClient(verify=self.ssl_verify)
            self.session = AsyncClient(verify=self.ssl_verify)
        # if (len(template_guids) == 0) or (len(integration_guids) == 0):
        #     self.build_global_guid_lists()
        #     self.template_guids = template_guids
        #     self.integration_guids = integration_guids

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.session.aclose()
        if exc_type is not None:
            self.exc_type = exc_type
            self.exc_val = exc_val
            self.exc_tb = exc_tb

        return False  # allows exceptions to propagate

    async def _async_close_session(self) -> None:
        """Close the session"""
        await self.session.aclose()

    def close_session(self) -> None:
        """Close the session"""
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self._async_close_session())
        return

    async def _async_create_egeria_bearer_token(self, user_id: str = None, password: str = None) -> str:
        """ Create and set an Egeria Bearer Token for the user. Async version
            Parameters
            ----------
            user_id : str, opt
                The user id to authenticate with. If None, then user_id from class instance used.
            password : str, opt
                The password for the user. If None, then user_pwd from class instance is used.

            Returns
            -------
            token
                The bearer token for the specified user.

            Raises
            ------
            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action
            Notes
            -----
            This routine creates a new bearer token for the user and updates the object with it.
            It uses Egeria's mechanisms to create a token. This is useful if an Egeria token expires.
            A bearer token from another source can be set with the set_bearer_token() method.

            """
        if user_id is None:
            validate_user_id(self.user_id)
            user_id = self.user_id
        if password is None:
            validate_name(self.user_pwd)
            password = self.user_pwd

        url = f"{self.platform_url}/api/token"
        data = {
            "userId": user_id,
            "password": password
        }
        async with AsyncClient(verify=self.ssl_verify) as client:
            try:
                response = await client.post(url, json=data, headers=self.headers)
                token = response.text
            except httpx.HTTPError as e:
                print(e)
                return "FAILED"

        if token:
            self.token_src = 'Egeria'
            self.headers["Authorization"] = f"Bearer {token}"
            self.text_headers["Authorization"] = f"Bearer {token}"
            return token
        else:
            raise InvalidParameterException("No token returned - request issue")

    def create_egeria_bearer_token(self, user_id: str = None, password: str = None) -> str:
        """ Create and set an Egeria Bearer Token for the user
        Parameters
        ----------
        user_id : str
            The user id to authenticate with.
        password : str
            The password for the user.

        Returns
        -------
        token
            The bearer token for the specified user.

        Raises
        ------
        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action
        Notes
        -----
        This routine creates a new bearer token for the user and updates the object with it.
        It uses Egeria's mechanisms to create a token. This is useful if an Egeria token expires.
        A bearer token from another source can be set with the set_bearer_token() method.

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(self._async_create_egeria_bearer_token(user_id, password))
        return response

    async def _async_refresh_egeria_bearer_token(self) -> None:
        """
        Refreshes the Egeria bearer token. Async version.

        This method is used to refresh the bearer token used for authentication with Egeria. It checks if the token
        source is 'Egeria', and if the user ID and password are valid. If all conditions are met, it calls the
        `create_egeria_bearer_token` method to create a new bearer token. Otherwise,
        it raises an `InvalidParameterException`.

        Parameters:

        Returns:
            None

        Raises:
            InvalidParameterException: If the token source is invalid.
        """
        if (self.token_src == 'Egeria') and validate_user_id(self.user_id) and validate_name(self.user_pwd):
            await self._async_create_egeria_bearer_token(self.user_id, self.user_pwd)
        else:
            raise InvalidParameterException("Invalid token source")

    def refresh_egeria_bearer_token(self) -> None:
        """
        Refreshes the Egeria bearer token.

        This method is used to refresh the bearer token used for authentication with Egeria. It checks if the token
        source is 'Egeria', and if the user ID and password are valid. If all conditions are met, it calls the
        `create_egeria_bearer_token` method to create a new bearer token. Otherwise,
        it raises an `InvalidParameterException`.

        Parameters:

        Returns:
            None

        Raises:
            InvalidParameterException: If the token source is invalid.
            PropertyServerException
                Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
                The principle specified by the user_id does not have authorization for the requested action
        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self._async_refresh_egeria_bearer_token())
        return

    def set_bearer_token(self, token: str) -> None:
        """ Retrieve and set a Bearer Token
        Parameters
        ----------
        token: str
        A bearer token supplied to the method.

        Returns
        -------
        None
            This method does not return anything.

        Raises
        ------
        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action
        Notes
        -----
        This routine sets the bearer token for the current object. The user is responsible for providing the token.

        """
        validate_name(token)
        self.headers["Authorization"] = f"Bearer {token}"
        self.text_headers["Authorization"] = f"Bearer {token}"

    def get_token(self) -> str:
        """ Retrieve and return the bearer token   """
        return self.text_headers["Authorization"]

    def make_request(self, request_type: str, endpoint: str, payload: str | dict = None,
                     time_out: int = 30) -> Response | str:
        """ Make a request to the Egeria API"""
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(self._async_make_request(request_type, endpoint,
                                                                    payload, time_out))
        return response

    async def _async_make_request(self, request_type: str, endpoint: str, payload: str | dict = None,
                                  time_out: int = 30) -> Response | str:
        """  Make a request to the Egeria API - Async Version
        Function to make an API call via the self.session Library. Raise an exception if the HTTP response code
        is not 200/201. IF there is a REST communication exception, raise InvalidParameterException.

        :param request_type: Type of Request.
               Supported Values - GET, POST, (not PUT, PATCH, DELETE).
               Type - String
        :param endpoint: API Endpoint. Type - String
        :param payload: API Request Parameters or Query String.
               Type - String or Dict
        :return: Response. Type - JSON Formatted String

        """
        class_name = __class__.__name__
        calling_frame = inspect.currentframe().f_back
        caller_method = inspect.getframeinfo(calling_frame).function

        try:
            response = ""
            if request_type == "GET":
                response = await self.session.get(endpoint, params=payload, headers=self.headers, timeout=time_out)

            elif request_type == "POST":
                if payload is None:
                    response = await self.session.post(endpoint, headers=self.headers, timeout=time_out)
                elif type(payload) is str:
                    response = await self.session.post(endpoint, headers=self.text_headers, data=payload,
                                                       timeout=time_out)
                else:
                    response = await self.session.post(endpoint, headers=self.headers,
                                                       json=payload, timeout=time_out)

            elif request_type == "POST-DATA":
                if True:
                    response = await self.session.post(endpoint, headers=self.headers,
                                                       data=payload, timeout=time_out)
            elif request_type == "DELETE":
                if True:
                    response = await self.session.delete(endpoint, headers=self.headers, timeout=time_out)

            status_code = response.status_code

            if status_code in (200, 201):
                # Success
                if is_json(response.text):
                    # now look at the response itself and throw an exception if an issue
                    related_code = response.json().get("relatedHTTPCode")
                    if related_code == 200:
                        return response
                    else:
                        exc_message = response.text
                        raise InvalidParameterException(exc_message)
                else:
                    msg = OMAGCommonErrorCode.CLIENT_SIDE_REST_API_ERROR.value[
                        "message_template"
                    ].format(
                        "**invalid JSON response - check parameters**",
                        caller_method,
                        class_name,
                        endpoint,
                        OMAGCommonErrorCode.CLIENT_SIDE_REST_API_ERROR.value[
                            "message_id"
                        ],
                    )
                    exc_msg = json.dumps(
                        {
                            "class": "VoidResponse",
                            "relatedHTTPCode": response.status_code,
                            "exceptionClassName": "InvalidParameterException",
                            "actionDescription": caller_method,
                            "exceptionErrorMessage": msg,
                            "exceptionErrorMessageId": OMAGCommonErrorCode.CLIENT_SIDE_REST_API_ERROR.value[
                                "message_id"
                            ],
                            "exceptionErrorMessageParameters": [
                                endpoint,
                                self.server_name,
                                self.user_id,
                            ],
                            "exceptionSystemAction": OMAGCommonErrorCode.CLIENT_SIDE_REST_API_ERROR.value[
                                "system_action"
                            ],
                            "exceptionUserAction": OMAGCommonErrorCode.CLIENT_SIDE_REST_API_ERROR.value[
                                "user_action"
                            ],
                            "exceptionProperties": {
                                "endpoint": endpoint,
                                "server": self.server_name,
                                "user_id": self.user_id,
                            },
                        }
                    )
                    raise InvalidParameterException(exc_msg)

            if response.status_code in (400, 401, 403, 404, 405, 415):
                # 4xx are client side errors - 400 bad request, 401 unauthorized
                msg = OMAGCommonErrorCode.CLIENT_SIDE_REST_API_ERROR.value[
                    "message_template"
                ].format(
                    str(response.status_code),
                    caller_method,
                    class_name,
                    endpoint,
                    OMAGCommonErrorCode.CLIENT_SIDE_REST_API_ERROR.value["message_id"],
                )
                exc_msg = json.dumps(
                    {
                        "class": "VoidResponse",
                        "relatedHTTPCode": response.status_code,
                        "exceptionClassName": "InvalidParameterException",
                        "actionDescription": caller_method,
                        "exceptionErrorMessage": msg,
                        "exceptionErrorMessageId": OMAGCommonErrorCode.CLIENT_SIDE_REST_API_ERROR.value[
                            "message_id"
                        ],
                        "exceptionErrorMessageParameters": [
                            endpoint,
                            self.server_name,
                            self.user_id,
                        ],
                        "exceptionSystemAction": OMAGCommonErrorCode.CLIENT_SIDE_REST_API_ERROR.value[
                            "system_action"
                        ],
                        "exceptionUserAction": OMAGCommonErrorCode.CLIENT_SIDE_REST_API_ERROR.value[
                            "user_action"
                        ],
                        "exceptionProperties": {
                            "endpoint": endpoint,
                            "server": self.server_name,
                            "user_id": self.user_id,
                        },
                    }
                )
                if response.status_code in (401, 403, 405):
                    raise UserNotAuthorizedException(exc_msg)
                else:
                    raise InvalidParameterException(exc_msg)

            elif response.status_code in (500, 501, 502, 503, 504):
                # server errors
                msg = OMAGCommonErrorCode.EXCEPTION_RESPONSE_FROM_API.value[
                          "message_template"
                      ].format(
                    str(response.status_code),
                    caller_method,
                    endpoint,
                    OMAGCommonErrorCode.EXCEPTION_RESPONSE_FROM_API.value["message_id"],
                ) + "==>System reports:'" + response.reason_phrase + "'"
                exc_msg = json.dumps(
                    {
                        "class": "VoidResponse",
                        "relatedHTTPCode": response.status_code,
                        "exceptionClassName": "PropertyServerException",
                        "actionDescription": caller_method,
                        "exceptionErrorMessage": msg,
                        "exceptionErrorMessageId": OMAGCommonErrorCode.CLIENT_SIDE_REST_API_ERROR.value[
                            "message_id"
                        ],
                        "exceptionErrorMessageParameters": [
                            endpoint,
                            self.server_name,
                            self.user_id,
                        ],
                        "exceptionSystemAction": OMAGCommonErrorCode.CLIENT_SIDE_REST_API_ERROR.value[
                            "system_action"
                        ],
                        "exceptionUserAction": OMAGCommonErrorCode.CLIENT_SIDE_REST_API_ERROR.value[
                            "user_action"
                        ],
                        "exceptionProperties": {
                            "endpoint": endpoint,
                            "server": self.server_name,
                            "user_id": self.user_id,
                        },
                    }
                )
                raise PropertyServerException(exc_msg)

        except InvalidParameterException:
            raise
        except UserNotAuthorizedException:
            raise
        except (
                httpx.NetworkError,
                httpx.ProtocolError,
                httpx.HTTPStatusError,
                httpx.TimeoutException,
        ) as e:
            if type(response) is str:
                reason = response
            else:
                reason = response.reason_phrase

            msg = OMAGCommonErrorCode.CLIENT_SIDE_REST_API_ERROR.value[
                      "message_template"
                  ].format(
                e.args[0],
                caller_method,
                class_name,
                endpoint,
                OMAGCommonErrorCode.CLIENT_SIDE_REST_API_ERROR.value["message_id"],
            ) + "==>System reports:'" + reason + "'"
            exc_msg = json.dumps(
                {
                    "class": "VoidResponse",
                    "relatedHTTPCode": 400,
                    "exceptionClassName": "InvalidParameterException",
                    "actionDescription": caller_method,
                    "exceptionErrorMessage": msg,
                    "exceptionErrorMessageId": OMAGCommonErrorCode.CLIENT_SIDE_REST_API_ERROR.value[
                        "message_id"
                    ],
                    "exceptionErrorMessageParameters": endpoint,
                    "exceptionSystemAction": OMAGCommonErrorCode.CLIENT_SIDE_REST_API_ERROR.value[
                        "system_action"
                    ],
                    "exceptionUserAction": OMAGCommonErrorCode.CLIENT_SIDE_REST_API_ERROR.value[
                        "user_action"
                    ],
                    "exceptionProperties": {"endpoint": endpoint},
                }
            )
            raise InvalidParameterException(exc_msg)

    def build_global_guid_lists(self) -> None:
        global template_guids, integration_guids

        self.create_egeria_bearer_token(self.user_id, self.user_pwd)
        # get all technology types
        url = (f"{self.platform_url}/servers/{self.server_name}/api/open-metadata/automated-curation/technology-types/"
               f"by-search-string?startFrom=0&pageSize=0&startsWith=false&"
               f"endsWith=false&ignoreCase=true")
        body = {"filter": ""}

        response = self.make_request("POST", url, body)
        tech_types = response.json().get("elements", "no tech found")
        if type(tech_types) is list:
            for tech_type in tech_types:
                # get tech type details
                display_name = tech_type["name"]

                url = f"{self.platform_url}/servers/{self.server_name}/api/open-metadata/automated-curation/technology-types/by-name"
                body = {"filter": display_name}
                response = self.make_request("POST", url, body)
                details = response.json().get("element","no type found")
                if type(details) is str:
                    continue
                # get templates and update the template_guids global
                templates = details.get("catalogTemplates", "Not Found")
                if type(templates) is str:
                    template_guids[display_name]= None
                else:
                    for template in templates:
                        template_name = template.get("name", None)
                        template_guid = template["relatedElement"]["guid"]
                        template_guids[template_name] = template_guid
                        # print(f"Added {template_name} template with GUID {template_guids[template_name]}")

                # Now find the integration connector guids
                resource_list = details.get('resourceList', ' ')
                if type(resource_list) is str:
                    integration_guids[display_name] = None
                else:
                    for resource in resource_list:
                        resource_guid = resource['relatedElement']['guid']
                        resource_type = resource['relatedElement']['type']['typeName']
                        if resource_type == "IntegrationConnector":
                            integration_guids[display_name] = resource_guid
                            # print(f"Added {display_name} integration connector with GUID {integration_guids[display_name]}")



if __name__ == "__main__":
    try:
        connection = Client(
            "active-metadata-store", "https://127.0.0.1:9443", "garygeeke", "foo"
        )
    except Exception as e:
        print(e)



#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.


A command line interface for Egeria Catalog User functions

This is an emerging capability based on the **click** package. Feedback welcome!

"""
import click
from trogon import tui

from examples.widgets.cat.get_asset_graph import asset_viewer
from examples.widgets.cat.get_collection import collection_viewer
from examples.widgets.cat.get_tech_type_elements import tech_viewer
from examples.widgets.cat.get_tech_type_template import template_viewer
from examples.widgets.cat.list_assets import display_assets
from examples.widgets.cat.list_glossary import display_glossary_terms
from examples.widgets.cat.list_tech_types import display_tech_types
from examples.widgets.cat.list_projects import display_project_list
from examples.widgets.cat.list_todos import display_to_dos as list_todos
from examples.widgets.cat.get_project_structure import project_structure_viewer
from examples.widgets.cat.get_project_dependencies import project_dependency_viewer
from examples.widgets.cat.list_cert_types import display_certifications
from examples.widgets.cat.list_relationships import list_relationships
from examples.widgets.cat.list_user_ids import list_user_ids

# from pyegeria import ServerOps
from examples.widgets.cli.ops_config import Config


# class Config(object):
#     def __init__(self, server: str = None, url: str = None, userid:str = None, password:str = None,
#                  timeout:int = 30, paging: bool = False):
#         self.server = server
#         self.url = url
#         self.userid = userid
#         self.password = password
#         self.timeout = timeout
#         self.paging = paging
#
#
# pass_config = click.make_pass_decorator(Config)

# @tui
# @tui('menu','menu','A textual command line interface')
@tui()
@click.version_option("0.0.1", prog_name="egeria_ops")
@click.group()
@click.option('--server', default='active-metadata-store', envvar='EGERIA_METADATA_STORE',
              help='Egeria metadata store to work with')
@click.option('--url', default='https://localhost:9443', envvar='EGERIA_PLATFORM_URL',
              help='URL of Egeria metadata store platform to connect to')
@click.option('--integration-daemon', default='integration-daemon', envvar='EGERIA_INTEGRATION_DAEMON',
              help='Egeria integration daemon to work with')
@click.option('--integration_daemon_url', default='https://localhost:9443', envvar='EGERIA_INTEGRATION_DAEMON_URL',
              help='URL of Egeria integration daemon platform to connect to')
@click.option('--view_server', default='view-server', envvar='EGERIA_VIEW_SERVER',
              help='Egeria view server to work with')
@click.option('--view_server_url', default='https://localhost:9443', envvar='EGERIA_VIEW_SERVER_URL',
              help='URL of Egeria view server platform to connect to')
@click.option('--engine_host', default='engine-host', envvar='EGERIA_ENGINE_HOST',
              help='Egeria engine host to work with')
@click.option('--engine_host_url', default='https://localhost:9443', envvar='EGERIA_ENGINE_HOST_URL',
              help='URL of Egeria engine host platform to connect to')
@click.option('--admin_user', default='garygeeke', envvar='EGERIA_ADMIN_USER', help='Egeria admin user')
@click.option('--admin_user_password', default='secret', envvar='EGERIA_ADMIN_PASSWORD',
              help='Egeria admin password')
@click.option('--userid', default='erinoverview', envvar='EGERIA_USER', help='Egeria user')
@click.option('--password', default='secret', envvar='EGERIA_PASSWORD',
              help='Egeria user password')
@click.option('--timeout', default=60, help='Number of seconds to wait')
@click.option('--verbose', is_flag=True, default=False, help='Enable verbose mode')
@click.option('--paging', is_flag=True, default=False, help='Enable paging snapshots vs live updates')
@click.option('--jupyter', is_flag=True, default=False, envvar='EGERIA_JUPYTER',
              help='Enable for rendering in a Jupyter terminal')
@click.option('--width', default=200, envvar='EGERIA_WIDTH', help='Screen width, in characters, to use')
@click.pass_context
def cli(ctx, server, url, view_server, view_server_url, integration_daemon, integration_daemon_url,
        engine_host, engine_host_url, admin_user, admin_user_password, userid, password, timeout, paging,
        verbose, jupyter, width):
    """An Egeria Command Line interface for Operations """
    ctx.obj = Config(server, url, view_server, view_server_url, integration_daemon,
                     integration_daemon_url, engine_host, engine_host_url,
                     admin_user, admin_user_password, userid, password,
                     timeout, paging, verbose, jupyter, width)
    ctx.max_content_width = 200
    ctx.ensure_object(Config)
    if verbose:
        click.echo(f"we are in verbose mode - server is {server}")


@cli.group("show")
@click.pass_context
def show(ctx):
    """Display an Egeria Object"""
    pass


@show.command('tech-types')
@click.option('--tech_type', default='*', help='Tech type to search for')
@click.pass_context
def show_tech_types(ctx, tech_type):
    """List deployed technology types"""
    c = ctx.obj
    display_tech_types(tech_type, c.view_server, c.view_server_url,
                       c.userid, c.password)


@show.command('tech-type-elements')
@click.option('--tech_type', default='PostgreSQL Server', help='Specific tech type to get elements for')
@click.pass_context
def show_tech_type_elements(ctx, tech_type):
    """List technology type elements"""
    c = ctx.obj
    tech_viewer(tech_type, c.view_server, c.view_server_url,
                c.userid, c.password)


@show.command('tech-type-templates')
@click.option('--tech-type', default='PostgreSQL Server', help='Specific tech type to get elements for')
@click.pass_context
def show_tech_type_templates(ctx, tech_type):
    """List technology type templates"""
    c = ctx.obj
    template_viewer(tech_type, c.view_server, c.view_server_url, c.userid,
                    c.password, c.jupyter, c.width)


@show.command('assets')
@click.argument('search-string')
@click.pass_context
def show_assets(ctx, search_string):
    """Find and display assets

    Usage: show assets <search-string>

           search-string must be greater than four characters.
    """
    c = ctx.obj
    display_assets(search_string, c.view_server, c.view_server_url, c.userid,
                   c.password, 60,c.jupyter, c.width)


@show.command('glossary-terms')
@click.option('--search-string', default='*',
              help='List glossary terms similar to search string - minimum of 4 characters')
@click.option('--glossary_guid', default=None, help='Optionally restrict search to glossary with the specified guid')
@click.pass_context
def show_terms(ctx, search_string, glossary_guid):
    """Find and display glossary terms"""
    c = ctx.obj
    display_glossary_terms(search_string, glossary_guid, c.view_server, c.view_server_url, c.userid,
                           c.password, c.jupyter, c.width)


@show.command('asset-graph')
@click.argument('asset_guid', nargs=1)
@click.pass_context
def show_asset_graph(ctx, asset_guid):
    """Display a tree graph of information about an asset

    Usage: show asset-graph <asset-guid>

           asset-guid must be a valid asset guid. These can be found through other commands such as 'show tech-type-elements'

    """
    c = ctx.obj
    asset_viewer(asset_guid, c.view_server, c.view_server_url, c.userid,
                 c.password, c.jupyter, c.width)


@show.command('collection')
@click.option('--root_collection', default='Root Sustainability Collection',
              help='View of tree of collections from a given root')
@click.pass_context
def show_asset_graph(ctx, root_collection):
    """Display a tree graph of information about an asset """
    c = ctx.obj
    collection_viewer(root_collection, c.view_server, c.view_server_url, c.userid,
                      c.password, c.jupyter, c.width)


@show.command('projects')
@click.option('--search-string', default='*',
              help='List Projects by Search String')
@click.pass_context
def show_projects(ctx, search_string):
    """Display a list of Egeria projects """
    c = ctx.obj
    display_project_list(search_string, c.view_server, c.view_server_url, c.userid,
                      c.password, False,c.jupyter, c.width)

@show.command('certification-types')
@click.option('--search-string', default = 'CertificationType', help="")
@click.pass_context
def show_certification_types(ctx, search_string):
    """Show certification types
        - generally stay with the default..
    """
    c = ctx.obj
    display_certifications(search_string, c.view_server, c.view_server_url, c.userid,
                           c.password, c.timeout, c.jupyter, c.width)

@show.command('project-structure')
@click.option('--project', default = 'Clinical Trials Management',
              help="Enter the root project to start from")
@click.pass_context
def show_project_structure(ctx, project):
    """Show the organization structure of the project starting from a root project"""
    c = ctx.obj
    project_structure_viewer(project, c.view_server, c.view_server_url, c.userid,
                           c.password, c.jupyter, c.width, c.timeout)

@show.command('project-dependencies')
@click.option('--project', default = 'Clinical Trials Management',
              help="Enter the root project to start from")
@click.pass_context
def show_project_dependencies(ctx, project):
    """Show the dependencies of a project starting from a root project"""
    c = ctx.obj
    project_dependency_viewer(project, c.view_server, c.view_server_url, c.userid,
                           c.password, c.jupyter, c.width, c.timeout)



@show.command('relationships')
@click.option('--relationship', default = 'Certification',
              help="Relationship type name to search for.")
@click.pass_context
def show_relationships(ctx, relationship):
    """Show the structure of the project starting from a root project"""
    c = ctx.obj
    list_relationships(relationship, c.view_server, c.view_server_url, c.userid,
                           c.password, c.timeout, c.jupyter, c.width)



@show.command('to-dos')
@click.option('--search-string', default='*',
              help='View the list of To-Do items')
@click.option('--status', type=click.Choice(['OPEN','IN_PROGRESS','WAITING','COMPLETE', 'ABANDONED', 'None'],
               case_sensitive='False'), help = 'Enter an optional status filter', required=False, default=None)
@click.pass_context
def show_todos(ctx, search_string, status):
    """Display a tree graph of information about an asset """
    c = ctx.obj
    list_todos(search_string, status, c.view_server, c.view_server_url, c.userid,
                   c.password, c.jupyter, c.width)

@show.command('user-ids')
@click.pass_context
def show_todos(ctx):
    """Display a tree graph of information about an asset """
    c = ctx.obj
    list_user_ids( c.view_server, c.view_server_url, c.userid,
                   c.password, c.jupyter, c.width)

#
#  Tell
#

@cli.group('tell')
@click.pass_context
def tell(ctx):
    """Perform actions an Egeria Objects"""
    pass


@tell.group('survey')
@click.pass_context
def survey(ctx):
    """Refresh the specified integration connector or ALL connectors if not specified"""
    c = ctx.obj
    pass


@survey.command('survey-uc-server')
@click.pass_context
@click.option('--uc_endpoint', default='https://localhost:8080',
              help="Endpoint of the Unity Catalog Server to Survey")
def survey_uc_server(ctx, uc_endpoint):
    """Survey the Unity Catalog server at the given endpoint"""
    c = ctx.obj
    pass
    # restart_connector(connector, c.integration_daemon, c.integration_daemon_url,
    #                   c.userid, c.password)


if __name__ == '__main__':
    cli()



"""
PDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

    Maintain and explore the contents of nested collections.

"""
import asyncio

# import json
from pyegeria._client import Client
from pyegeria._globals import enable_ssl_check, max_paging_size


class ValidMetadataManager(Client):
    """ The Valid Metadata OMVS provides APIs for retrieving and updating lists of valid metadata values.
        For more details see: https://egeria-project.org/guides/planning/valid-values/overview/

    Attributes:

        server_name: str
            The name of the View Server to connect to.
        platform_url : str
            URL of the server platform to connect to
        user_id : str
            The identity of the user calling the method - this sets a default optionally used by the methods
            when the user doesn't pass the user_id on a method call.
        user_pwd: str
            The password associated with the user_id. Defaults to None

     """

    def __init__(
            self,
            server_name: str,
            platform_url: str,
            token: str = None,
            user_id: str = None,
            user_pwd: str = None,
            verify_flag: bool = enable_ssl_check,
    ):
        self.command_base: str = f"/api/open-metadata/valid-metadata"
        self.page_size = max_paging_size
        Client.__init__(self, server_name, platform_url, user_id=user_id, token=token)

    async def _async_setup_valid_metadata_value(self, property_name: str, type_name: str, body: dict,
                                                server: str = None):
        """ Create or update the valid value for a particular open metadata property name. If the typeName is null,
        this valid value applies to properties of this name from all types. The valid value is stored in the
        preferredValue property. If a valid value is already set up for this property (with overlapping effective dates)
        then the valid value is updated. Async Version.

        Parameters
        ----------
        property_name : str
            The name of the property for which the valid metadata value is being set up.
        type_name : str
            The name of the type for the valid metadata value.
        body : dict
            The body of the request containing the details of the valid metadata value.
        server : str, optional
            The name of the server where the valid metadata value is being set up.
            If not provided, the default server name will be used.

        Returns
        -------

        Notes
        -----

        Payload structure similar to:
        {
          "displayName": "",
          "description": "",
          "preferredValue": "",
          "dataType": "",
          "isCaseSensitive": false,
          "isDeprecated" : false
        }
        """
        server = self.server_name if server is None else server

        url = (f"{self.platform_url}/servers/{server}/api/open-metadata/valid-metadata/setup-value/{property_name}?"
               f"typeName={type_name}")

        await self._async_make_request("POST", url, body)
        return

    def setup_valid_metadata_value(self, property_name: str, type_name: str, body: dict,
                                   server: str = None):
        """ Create or update the valid value for a particular open metadata property name. If the typeName is null,
        this valid value applies to properties of this name from all types. The valid value is stored in the
        preferredValue property. If a valid value is already set up for this property (with overlapping effective dates)
        then the valid value is updated.

        Parameters
        ----------
        property_name : str
            The name of the property for which the valid metadata value is being set up.
        type_name : str
            The name of the type for the valid metadata value.
        body : dict
            The body of the request containing the details of the valid metadata value.
        server : str, optional
            The name of the server where the valid metadata value is being set up.
            If not provided, the default server name will be used.

        Returns
        -------
        str
            The GUID of the valid metadata value if it was successfully set up, or "GUID failed to be returned"
            if the GUID was not returned in the response.

        Notes
        -----

        Payload structure similar to:
        {
          "displayName": "",
          "description": "",
          "preferredValue": "",
          "dataType": "",
          "isCaseSensitive": false,
          "isDeprecated" : false,
          "additionalProperties": {
            "colour": "purple"
            }
        }
        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self._async_setup_valid_metadata_value(property_name, type_name,
                                                                       body, server))
        return

    async def _async_setup_valid_metadata_map_name(self, property_name: str, type_name: str, body: dict,
                                                   server: str = None):
        """ Create or update the valid value for a name that can be stored in a particular open metadata property name.
            This property is of type map from name to string. The mapName is stored in the preferredValue property of
            validMetadataValue. If the typeName is null, this valid value applies to properties of this name from any
            open metadata type. If a valid value is already set up for this property (with overlapping effective dates)
            then the valid value is updated. Async Version.

            Parameters
            ----------
            property_name : str
                The name of the property to setup metadata map.
            type_name : str
                The type name of the property.
            body : dict
                The metadata map setup data.
            server : str, optional
                The name of the server to setup the metadata map. If not provided, the default server name will be used.

            Returns
            -------
            None
                This method does not return any value.

            Notes
            -----

            Body strycture similar to:

            {
              "displayName": "",
              "description": "",
              "preferredValue": "put mapName value here",
              "dataType": "",
              "isCaseSensitive": false,
              "isDeprecated" : false
            }

        """
        server = self.server_name if server is None else server

        url = (f"{self.platform_url}/servers/{server}/api/open-metadata/valid-metadata/setup-map-name/{property_name}?"
               f"typeName={type_name}")

        await self._async_make_request("POST", url, body)
        return

    async def _async_setup_valid_metadata_type_value(self, property_name: str, type_name: str, map_name: str,
                                                     server: str = None):
        pass

    async def _async_clear_valid_metadata_value(self, property_name: str, type_name: str, map_name: str,
                                                server: str = None):
        pass

    async def _async_clear_valid_metadata_map_value(self, property_name: str, type_name: str, map_name: str,
                                                    preferred_value: str, server: str = None):
        pass

    async def _async_validate_metadata_value(self, property_name: str, type_name: str, actual_value: str,
                                             server: str = None):
        pass

    async def _async_validate_metadata_map_name(self, property_name: str, type_name: str, map_name: str,
                                                server: str = None):
        pass

    async def _async_validate_metadata_map_value(self, property_name: str, type_name: str, actual_value: str,
                                                 server: str = None):
        pass

    async def _async_validate_metadata_map_name(self, property_name: str, type_name: str, map_name: str,
                                                server: str = None):
        pass

    async def _async_get_valid_metadata_value(self, property_name: str, type_name: str, preferred_value: str,
                                              server: str = None):
        pass

    async def _async_get_valid_metadata_map_name(self, property_name: str, type_name: str, map_name: str,
                                                 server: str = None):
        pass

    async def _async_get_valid_metadata_map_value(self, property_name: str, type_name: str, map_name: str,
                                                  preferred_value: str, server: str = None):
        pass

    async def _async_get_valid_metadata_values(self, property_name: str, type_name: str = None,
                                               server_name: str = None,
                                               start_value: int = 0, page_size: int = None) -> list | str:
        """ Retrieve list of values for the property. Async version.

            Parameters
            ----------
            property_name: str
                The property to query.
            type_name: str, opt
                The Open Metadata type to get the property values for. If not specified then all property values
                will be returned.
            server_name : str, optional
                The name of the server to  configure.
                If not provided, the server name associated with the instance is used.

            Returns
            -------
            List | str

            A list of collections linked off of the supplied element.

            Raises
            ------

            InvalidParameterException
              If the client passes incorrect parameters on the request - such as bad URLs or invalid values
            PropertyServerException
              Raised by the server when an issue arises in processing a valid request
            NotAuthorizedException
              The principle specified by the user_id does not have authorization for the requested action

                """
        if server_name is None:
            server_name = self.server_name
        if page_size is None:
            page_size = self.page_size

        url = (f"{self.platform_url}/servers/{server_name}{self.command_base}/get-valid-metadata-values/{property_name}"
               f"?typeName={type_name}&startFrom={start_value}&pageSize={page_size}")

        resp = await self._async_make_request("GET", url)
        return resp.json().get("elementList", "No elements found")

    def get_valid_metadata_values(self, property_name: str, type_name: str = None,
                                  server_name: str = None) -> list | str:
        """  Retrieve list of values for the property.

        Parameters
        ----------
        property_name: str
            The property to query.
        type_name: str, opt
            The Open Metadata type to get the property values for. If not specified then all property values
            will be returned.
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.

        Returns
        -------
        List | str

        A list of collections linked off of the supplied element.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        loop = asyncio.get_event_loop()
        resp = loop.run_until_complete(self._async_get_valid_metadata_values(property_name, type_name,
                                                                             server_name))
        return resp

    async def _async_get_valid_metadata_value(self, property_name: str, type_name: str, preferred_value: str,
                                              server_name: str = None) -> list | str:
        if server_name is None:
            server_name = self.server_name

        url = (f"{self.platform_url}/servers/{server_name}{self.command_base}/get-value/{property_name}"
               f"?typeName={type_name}&preferredValue={preferred_value}")

        resp = await self._async_make_request("GET", url)
        return resp.json()

    def get_valid_metadata_value(self, property_name: str, type_name: str, preferred_value: str,
                                 server_name: str = None) -> list | str:
        """  Retrieve details of a specific valid value for a property.

        Parameters
        ----------
        property_name: str
            The property name of the valid metadata value to retrieve
        type_name: str
            Type of the metadata value to retrieve
        preferred_value: str
            The preferred value of the valid metadata value to retrieve
        server_name : str, opt
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.

        Returns
        -------
        List | str

        A list of collections linked off of the supplied element.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        loop = asyncio.get_event_loop()
        resp = loop.run_until_complete(self._async_get_valid_metadata_value(property_name, type_name,
                                                                            preferred_value, server_name)),
        return resp

    async def _async_get_consistent_metadata_values(self, property_name: str, type_name: str, map_name: str,
                                                    preferred_value: str, server_name: str = None,
                                                    start_from: int = 0, page_size: int = None) -> list | str:
        """  Retrieve all the consistent valid values for the requested property. Async version.

        Parameters
        ----------
        property_name : str
            The name of the property to retrieve the valid values for.
        type_name : str
            The open metadata type that the property is associated with.
        map_name : str
            A valid map name that associates a property with a value.
        preferred_value : str

        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.
        start_from: int, [default=0], optional
                    When multiple pages of results are available, the page number to start from.
        page_size: int, [default=None]
            The number of items to return in a single page. If not specified, the default will be taken from
            the class instance.
        Returns
        -------
        List | str

        A list of collections linked off of the supplied element.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        if page_size is None:
            page_size = self.page_size

        url = (f"{self.platform_url}/servers/{server_name}{self.command_base}/{property_name}/"
               f"consistent-metadata-values?typeName={type_name}&mapName={map_name}&preferredValue={preferred_value}"
               f"&startFrom={start_from}&pageSize={page_size}")

        resp = await self._async_make_request("GET", url)
        return resp.json()

    def get_consistent_metadata_values(self, property_name: str, type_name: str, map_name: str,
                                       preferred_value: str, server_name: str = None,
                                       start_from: int = 0, page_size: int = None) -> list | str:
        """  Retrieve all the consistent valid values for the requested property.

        Parameters
        ----------
        property_name : str
            The name of the property to retrieve the valid values for.
        type_name : str
            The open metadata type that the property is associated with.
        map_name : str
            A valid map name that associates a property with a value.
        preferred_value : str

        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.
        start_from: int, [default=0], optional
                    When multiple pages of results are available, the page number to start from.
        page_size: int, [default=None]
            The number of items to return in a single page. If not specified, the default will be taken from
            the class instance.
        Returns
        -------
        List | str

        A list of collections linked off of the supplied element.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        loop = asyncio.get_event_loop()
        resp = loop.run_until_complete(
            self._async_get_consistent_metadata_values(property_name, type_name,
                                                       map_name, preferred_value,
                                                       server_name, start_from, page_size)
        )
        return resp

    #
    # Get all ...
    #
    async def _async_get_all_entity_types(self, server_name: str = None) -> list | str:
        """ Returns the list of different types of metadata organized into two groups.  The first are the
            attribute type definitions (AttributeTypeDefs).  These provide types for attributes in full
            type definitions.  Full type definitions (TypeDefs) describe types for entities, relationships
            and classifications. Async version.

        Parameters
        ----------
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.

        Returns
        -------
        List | str

            A list of all entity types.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name

        url = f"{self.platform_url}/servers/{server_name}{self.command_base}/open-metadata-types"

        resp = await self._async_make_request("GET", url)
        return resp.json().get("typeDefs", "No TypeDefs Found")

    def get_all_entity_types(self, server_name: str = None) -> list | str:
        """ Returns the list of different types of metadata organized into two groups.  The first are the
            attribute type definitions (AttributeTypeDefs).  These provide types for attributes in full
            type definitions.  Full type definitions (TypeDefs) describe types for entities, relationships
            and classifications. Async version.

        Parameters
        ----------
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.

        Returns
        -------
        List | str

            A list of all entity types.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        loop = asyncio.get_event_loop()
        resp = loop.run_until_complete(
            self._async_get_all_entity_types(server_name))
        return resp

    async def _async_get_all_entity_defs(self, server_name: str = None) -> list | str:
        """ GReturns all the entity type definitions. Async version.

        Parameters
        ----------
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.

        Returns
        -------
        List | str

            A list of all entity types.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name

        url = f"{self.platform_url}/servers/{server_name}{self.command_base}/open-metadata-types/entity-defs"

        resp = await self._async_make_request("GET", url)
        return resp.json().get("typeDefs", "No TypeDefs Found")

    def get_all_entity_defs(self, server_name: str = None) -> list | str:
        """ Returns all the entity type definitions.

        Parameters
        ----------
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.

        Returns
        -------
        List | str

            A list of all entity types.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        loop = asyncio.get_event_loop()
        resp = loop.run_until_complete(
            self._async_get_all_entity_defs(server_name))
        return resp

    async def _async_get_all_relationship_defs(self, server_name: str = None) -> list | str:
        """ Returns all the relationship type definitions. Async version.

        Parameters
        ----------
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.

        Returns
        -------
        List | str

            A list of all entity types.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name

        url = f"{self.platform_url}/servers/{server_name}{self.command_base}/open-metadata-types/relationship-defs"

        resp = await self._async_make_request("GET", url)
        return resp.json().get("typeDefs", "No TypeDefs Found")

    def get_all_relationship_defs(self, server_name: str = None) -> list | str:
        """ Returns all the relationship type definitions.

        Parameters
        ----------
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.

        Returns
        -------
        List | str

            A list of all entity types.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        loop = asyncio.get_event_loop()
        resp = loop.run_until_complete(
            self._async_get_all_relationship_defs(server_name))
        return resp

    async def _async_get_all_classification_defs(self, server_name: str = None) -> list | str:
        """ Returns all the classification type definitions. Async version.

        Parameters
        ----------
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.

        Returns
        -------
        List | str

            A list of all entity types.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name

        url = f"{self.platform_url}/servers/{server_name}{self.command_base}/open-metadata-types/classification-defs"

        resp = await self._async_make_request("GET", url)
        return resp.json().get("typeDefs", "No TypeDefs Found")

    def get_all_classification_defs(self, server_name: str = None) -> list | str:
        """ Returns all the classification type definitions.

        Parameters
        ----------
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.

        Returns
        -------
        List | str

            A list of all entity types.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        loop = asyncio.get_event_loop()
        resp = loop.run_until_complete(
            self._async_get_all_classification_defs(server_name))
        return resp

    #
    # Get valid ...
    #
    async def _async_get_valid_relationship_types(self, entity_type: str, server_name: str = None) -> list | str:
        """  Returns all the TypeDefs for relationships that can be attached to the requested entity type.
            Async version.

        Parameters
        ----------
        entity_type : str
            The name of the entity type to retrieve the valid relationships for.
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.

        Returns
        -------
        List | str

            A list of TypeDefs that can be attached to the specified entity type.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name

        url = (f"{self.platform_url}/servers/{server_name}{self.command_base}/open-metadata-types/{entity_type}/"
               f"attached-relationships")

        resp = await self._async_make_request("GET", url)
        return resp.json().get("typeDefs", "No TypeDefs Found")

    def get_valid_relationship_types(self, entity_type: str, server_name: str = None) -> list | str:
        """  Returns all the TypeDefs for relationships that can be attached to the requested entity type.
                    Async version.

            Parameters
            ----------
            entity_type : str
                 The name of the entity type to retrieve the valid relationships for.
            server_name : str, optional
                The name of the server to  configure.
                If not provided, the server name associated with the instance is used.

            Returns
            -------
            List | str

                A list of TypeDefs that can be attached to the specified entity type.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        loop = asyncio.get_event_loop()
        resp = loop.run_until_complete(
            self._async_get_valid_relationship_types(entity_type,
                                                     server_name)
        )
        return resp

    async def _async_get_valid_classification_types(self, entity_type: str, server_name: str = None) -> list | str:
        """ Returns all the TypeDefs for classifications that can be attached to the requested entity type.
            Async version.

        Parameters
        ----------
        entity_type : str
            The name of the entity type to retrieve the classifications for.
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.

        Returns
        -------
        List | str

            A list of classifications that can be attached to the specified entity type.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name

        url = (f"{self.platform_url}/servers/{server_name}{self.command_base}/open-metadata-types/{entity_type}/"
               f"attached-classifications")

        resp = await self._async_make_request("GET", url)
        return resp.json().get("typeDefs", "No TypeDefs Found")

    def get_valid_classification_types(self, entity_type: str, server_name: str = None) -> list | str:
        """  Returns all the TypeDefs for relationships that can be attached to the requested entity type.
                    Async version.

            Parameters
            ----------
            entity_type : str
                The name of the entity type to retrieve the classifications for.
            server_name : str, optional
                The name of the server to  configure.
                If not provided, the server name associated with the instance is used.

            Returns
            -------
            List | str

                A list of classifications that can be attached to the specified entity type.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        loop = asyncio.get_event_loop()
        resp = loop.run_until_complete(
            self._async_get_valid_classification_types(entity_type,
                                                       server_name)
        )
        return resp

    async def _async_get_typedef_by_name(self, entity_type: str, server_name: str = None) -> dict | str:
        """ Return the TypeDef identified by the unique name.
            Async version.

        Parameters
        ----------
        entity_type : str
            The name of the entity type to retrieve the typedef for.
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.

        Returns
        -------
        dict | str

            The typedef associated with the type name

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name

        url = f"{self.platform_url}/servers/{server_name}{self.command_base}/open-metadata-types/name/{entity_type}"

        resp = await self._async_make_request("GET", url)
        return resp.json().get("typeDef", "No TypeDefs Found")

    def get_typedef_by_name(self, entity_type: str, server_name: str = None) -> dict | str:
        """ Return the TypeDef identified by the unique name.

        Parameters
        ----------
        entity_type : str
            The name of the entity type to retrieve the typedef for.
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.

        Returns
        -------
        dict | str

            The typedef associated with the type name

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action

        """
        loop = asyncio.get_event_loop()
        resp = loop.run_until_complete(
            self._async_get_typedef_by_name(entity_type, server_name))
        return resp



#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

Unit tests for the Utils helper functions using the Pytest framework.


A simple status display for Engine Actions
"""

import argparse
import json
import os
import sys
import time

from rich import box
from rich.console import Console
from rich.live import Live
from rich.markdown import Markdown
from rich.table import Table

from pyegeria import AutomatedCuration
from pyegeria import (
    InvalidParameterException,
    PropertyServerException,
    UserNotAuthorizedException,
    print_exception_response,
)

EGERIA_METADATA_STORE = os.environ.get("EGERIA_METADATA_STORE", "active-metadata-store")
EGERIA_KAFKA_ENDPOINT = os.environ.get("KAFKA_ENDPOINT", "localhost:9092")
EGERIA_PLATFORM_URL = os.environ.get("EGERIA_PLATFORM_URL", "https://localhost:9443")
EGERIA_VIEW_SERVER = os.environ.get("VIEW_SERVER", "view-server")
EGERIA_VIEW_SERVER_URL = os.environ.get(
    "EGERIA_VIEW_SERVER_URL", "https://localhost:9443"
)
EGERIA_ENGINE_HOST = os.environ.get("INTEGRATION_ENGINE_HOST", "engine-host")
EGERIA_ENGINE_HOST_URL = os.environ.get(
    "INTEGRATION_ENGINE_HOST_URL", "https://localhost:9443"
)
EGERIA_INTEGRATION_DAEMON = os.environ.get("INTEGRATION_DAEMON", "integration-daemon")
EGERIA_ADMIN_USER = os.environ.get("ADMIN_USER", "garygeeke")
EGERIA_ADMIN_PASSWORD = os.environ.get("ADMIN_PASSWORD", "secret")
EGERIA_USER = os.environ.get("EGERIA_USER", "erinoverview")
EGERIA_USER_PASSWORD = os.environ.get("EGERIA_USER_PASSWORD", "secret")
EGERIA_JUPYTER = bool(os.environ.get("EGERIA_JUPYTER", "False"))
EGERIA_WIDTH = int(os.environ.get("EGERIA_WIDTH", "200"))

disable_ssl_warnings = True


def display_engine_activity_c(
    server: str,
    url: str,
    user: str,
    user_pass: str,
    paging: bool,
    jupyter: bool = EGERIA_JUPYTER,
    width=EGERIA_WIDTH,
):
    g_client = AutomatedCuration(server, url, user, user_pwd=user_pass)

    def generate_table() -> Table:
        """Make a new table."""
        table = Table(
            title=f"Engine Action Status for Platform {url} @ {time.asctime()}",
            style="bold white on black",
            row_styles=["bold white on black"],
            header_style="white on dark_blue",
            title_style="bold white on black",
            caption_style="white on black",
            show_lines=True,
            box=box.ROUNDED,
            caption=f"Engine Status for Server '{server}' @ Platform - {url}",
            expand=True,
        )
        table.add_column("Requested Time")
        table.add_column("Core Info")

        # table.add_column("Request Type")
        table.add_column("Action Status")
        table.add_column("Target Element")
        table.add_column("Completion Time")
        table.add_column("Process Name")
        table.add_column("Completion Message")

        token = g_client.create_egeria_bearer_token()
        action_status = g_client.get_engine_actions()

        if type(action_status) is str:
            requested_time = " "
            start_time = " "
            completion_time = " "
            engine_name = " "
            request_type = " "
            action_status = " "
            target_element = " "
            process_name = " "
            completion_message = " "
        elif type(action_status) is list:
            sorted_action_status = sorted(
                action_status,
                key=lambda i: i.get("requestedTime", time.asctime()),
                reverse=True,
            )
            for action in sorted_action_status:
                requested_time = action.get("requestedTime", " ")
                start_time = action.get("startTime", " ")
                completion_time = action.get("completionTime", " ")

                engine_name = action["governanceEngineName"]
                request_type = action["requestType"]
                action_guid = action["elementHeader"]["guid"]
                if action["actionStatus"] in (
                    "REQUESTED",
                    "APPROVED",
                    "WAITING",
                    "ACTIVATING",
                ):
                    action_status = f"[yellow]{action['actionStatus']}"
                elif action["actionStatus"] in ("IN_PROGRESS", "ACTIONED"):
                    action_status = f"[green]{action['actionStatus']}"
                else:
                    action_status = f"[red]{action['actionStatus']}"

                targets = action.get("actionTargetElements", "Empty")
                if type(targets) is list:
                    tgt_tab = Table()
                    tgt_tab.add_column("name")
                    tgt_tab.add_column("guid", no_wrap=True)
                    tgt_tab.add_column("type_name")
                    targets_md = ""
                    for target in targets:
                        t_name = target["actionTargetName"]
                        t_guid = target["actionTargetGUID"]
                        t_type = target["targetElement"]["type"]["typeName"]
                        tgt_tab.add_row(t_name, t_guid, t_type)
                    # target_element = json.dumps(target[0]["targetElement"]["elementProperties"]["propertiesAsStrings"])
                    target_element = tgt_tab
                else:
                    target_element = " "

                process_name = action.get("processName", " ")
                completion_message = action.get("completionMessage", " ")

                core_info_md = (
                    f"* Start Time: {start_time}\n* Engine Name: {engine_name}\n* GUID: {action_guid}\n"
                    f"* Request TYpe: {request_type}"
                )
                core_info_out = Markdown(core_info_md)
                table.add_row(
                    requested_time,
                    core_info_out,
                    action_status,
                    target_element,
                    completion_time,
                    process_name,
                    completion_message,
                )
        else:
            print("Egeria integration daemon not running")
            sys.exit()

        return table

    try:
        if paging is True:
            console = Console(width=width, force_terminal=not jupyter)
            with console.pager():
                console.print(generate_table())
        else:
            with Live(
                generate_table(),
                refresh_per_second=1,
                screen=True,
                vertical_overflow="visible",
            ) as live:
                while True:
                    time.sleep(2)
                    live.update(generate_table())

    except (
        InvalidParameterException,
        PropertyServerException,
        UserNotAuthorizedException,
    ) as e:
        print_exception_response(e)
    except KeyboardInterrupt:
        pass
    finally:
        g_client.close_session()


def main_live():
    parser = argparse.ArgumentParser()
    parser.add_argument("--server", help="Name of the server to display status for")
    parser.add_argument("--url", help="URL Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    parser.add_argument("--password", help="User Password")

    args = parser.parse_args()

    server = args.server if args.server is not None else EGERIA_VIEW_SERVER
    url = args.url if args.url is not None else EGERIA_VIEW_SERVER_URL
    userid = args.userid if args.userid is not None else EGERIA_USER
    user_pass = args.password if args.password is not None else EGERIA_USER_PASSWORD

    display_engine_activity_c(
        server=server, url=url, user=userid, user_pass=user_pass, paging=False
    )


def main_paging():
    parser = argparse.ArgumentParser()
    parser.add_argument("--server", help="Name of the server to display status for")
    parser.add_argument("--url", help="URL Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    parser.add_argument("--password", help="User Password")

    args = parser.parse_args()

    server = args.server if args.server is not None else EGERIA_VIEW_SERVER
    url = args.url if args.url is not None else EGERIA_VIEW_SERVER_URL
    userid = args.userid if args.userid is not None else EGERIA_USER
    user_pass = args.password if args.password is not None else EGERIA_USER_PASSWORD

    display_engine_activity_c(
        server=server, url=url, user=userid, user_pass=user_pass, paging=True
    )


if __name__ == "__main__":
    main_live()

if __name__ == "__main_paging__":
    main_paging()



"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.



This script refreshed an integration daemon.

"""

import os
import argparse
import time
import click
from trogon import tui
from pyegeria import ServerOps, AutomatedCuration
from pyegeria._exceptions import (
    InvalidParameterException,
    PropertyServerException,
    UserNotAuthorizedException,
    print_exception_response,
)
EGERIA_METADATA_STORE = os.environ.get("EGERIA_METADATA_STORE", "active-metadata-store")
EGERIA_KAFKA_ENDPOINT = os.environ.get('KAFKA_ENDPOINT', 'localhost:9092')
EGERIA_PLATFORM_URL = os.environ.get('EGERIA_PLATFORM_URL', 'https://localhost:9443')
EGERIA_VIEW_SERVER = os.environ.get('VIEW_SERVER', 'view-server')
EGERIA_VIEW_SERVER_URL = os.environ.get('EGERIA_VIEW_SERVER_URL', 'https://localhost:9443')
EGERIA_INTEGRATION_DAEMON = os.environ.get('INTEGRATION_DAEMON', 'integration-daemon')
EGERIA_INTEGRATION_DAEMON_URL = os.environ.get('EGERIA_INTEGRATION_DAEMON_URL', 'https://localhost:9443')
EGERIA_ADMIN_USER = os.environ.get('ADMIN_USER', 'garygeeke')
EGERIA_ADMIN_PASSWORD = os.environ.get('ADMIN_PASSWORD', 'secret')
EGERIA_USER = os.environ.get('EGERIA_USER', 'erinoverview')
EGERIA_USER_PASSWORD = os.environ.get('EGERIA_USER_PASSWORD', 'secret')


@click.command('load-archive')
@click.option('--file', prompt= "Path on the Metadata Server of the archive file to load",
              default='content-packs/CocoComboArchive.omarchive',
              help='Full path on the Metadata Server to the archive file to load')
@click.option('--server', default = EGERIA_METADATA_STORE, help='Egeria metadata store to load')
@click.option('--url', default = EGERIA_PLATFORM_URL, help='URL of Egeria platform to connect to')
@click.option('--userid', default = EGERIA_ADMIN_USER, help='Egeria admin user')
@click.option('--password', default = EGERIA_ADMIN_PASSWORD, help='Egeria admin password')
@click.option('--timeout', default = 60, help = 'Number of seconds to wait')
def load_archive(file, server, url, userid, password, timeout):
    """Load an Open Metadata Archive"""

    try:

        s_client = ServerOps(server, url, userid, password)

        s_client.add_archive_file(file, server, timeout = timeout)

        click.echo(f"Loaded archive: {file}")


    except (InvalidParameterException, PropertyServerException) as e:
        print_exception_response(e)


if __name__ == "__main__":
    load_archive()


#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

Unit tests for the Utils helper functions using the Pytest framework.


A simple display for my profile
"""
import os
import argparse
import sys
import time

from rich import box
from rich.console import Console
from rich.prompt import Prompt
from rich.table import Table

from pyegeria import (
    InvalidParameterException,
    PropertyServerException,
    UserNotAuthorizedException,
    print_exception_response,
    AutomatedCuration
)

EGERIA_METADATA_STORE = os.environ.get("EGERIA_METADATA_STORE", "active-metadata-store")
EGERIA_KAFKA_ENDPOINT = os.environ.get('KAFKA_ENDPOINT', 'localhost:9092')
EGERIA_PLATFORM_URL = os.environ.get('EGERIA_PLATFORM_URL', 'https://localhost:9443')
EGERIA_VIEW_SERVER = os.environ.get('VIEW_SERVER', 'view-server')
EGERIA_VIEW_SERVER_URL = os.environ.get('EGERIA_VIEW_SERVER_URL', 'https://localhost:9443')
EGERIA_INTEGRATION_DAEMON = os.environ.get('INTEGRATION_DAEMON', 'integration-daemon')
EGERIA_ADMIN_USER = os.environ.get('ADMIN_USER', 'garygeeke')
EGERIA_ADMIN_PASSWORD = os.environ.get('ADMIN_PASSWORD', 'secret')
EGERIA_USER = os.environ.get('EGERIA_USER', 'erinoverview')
EGERIA_USER_PASSWORD = os.environ.get('EGERIA_USER_PASSWORD', 'secret')
EGERIA_JUPYTER = bool(os.environ.get('EGERIA_JUPYTER', 'False'))
EGERIA_WIDTH = int(os.environ.get('EGERIA_WIDTH', '200'))


disable_ssl_warnings = True


def display_tech_types(search_string:str, server: str, url: str, username: str, user_pass: str,
                       jupyter: bool = EGERIA_JUPYTER, width: int = EGERIA_WIDTH):
    a_client = AutomatedCuration(server, url, username)
    token = a_client.create_egeria_bearer_token(username, user_pass)
    tech_list = a_client.find_technology_types(search_string, page_size=0)

    def generate_table() -> Table:
        """Make a new table."""
        table = Table(
            title=f"Technology Types for: {url} @ {time.asctime()}",
            style="bold white on black",
            row_styles=["bold white on black"],
            header_style="white on dark_blue",
            title_style="bold white on black",
            caption_style="white on black",
            show_lines=True,
            box=box.ROUNDED,
            caption=f"Technology Types from Server '{server}' @ Platform - {url}",
            expand=True
        )

        table.add_column("Name")
        table.add_column("Qualified Name")
        table.add_column("Category")
        table.add_column("Description")


        name = " "
        description = " "
        version = " "
        super_type = " "
        if type(tech_list) is list:
            for item in tech_list:
                if 'deployedImplementationType' not in item['qualifiedName']:
                    continue
                qualified_name = item.get("qualifiedName", " ")
                name = item.get("name", "none")
                category = item.get("category", "none")
                description = item.get("description", "none")

                table.add_row(
                    name, qualified_name, category, description
                )
            return table
        else:
            print("Unknown technology type")
            sys.exit(1)

    try:
        console = Console(width=width, force_terminal=not jupyter)
        with console.pager(styles=True):
            console.print(generate_table())

    except (InvalidParameterException, PropertyServerException, UserNotAuthorizedException) as e:
        print_exception_response(e)
        assert e.related_http_code != "200", "Invalid parameters"
    finally:
        a_client.close_session()


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--server", help="Name of the server to display status for")
    parser.add_argument("--url", help="URL Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    parser.add_argument("--password", help="User Password")

    args = parser.parse_args()

    server = args.server if args.server is not None else EGERIA_VIEW_SERVER
    url = args.url if args.url is not None else EGERIA_PLATFORM_URL
    userid = args.userid if args.userid is not None else EGERIA_USER
    user_pass = args.password if args.password is not None else EGERIA_USER_PASSWORD

    try:
        search_string = Prompt.ask("Enter the technology you are searching for:", default="*")
        display_tech_types(search_string, server, url, userid, user_pass)
    except (KeyboardInterrupt):
        pass


if __name__ == "__main__":
    main()


"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

 OMAG Server operations.  These functions provide operations on a running OMAG server.

"""

import asyncio

from requests import Response

from pyegeria import Platform
from pyegeria._validators import validate_name


class ServerOps(Platform):
    """
    Client to issue operations on a running OMAG server.

    Attributes:
        server_name: str
            Name of the server to use.
        platform_url : str
            URL of the server platform to connect to
        user_id : str
            The identity of the user calling the method - this sets a default optionally used by the methods
            when the user doesn't pass the user_id on a method call.
        user_pwd: str
            The password associated with the user_id. Defaults to None

    """
    def __init__(
            self,
            server_name: str,
            platform_url: str,
            user_id: str,
            user_pwd: str = None,
            verify_flag: bool = False,
    ):
        Platform.__init__(self, server_name, platform_url, user_id, user_pwd, verify_flag)
        self.ops_command_root = f"{self.platform_url}/open-metadata/server-operations/users/{user_id}"

    async def _async_get_active_configuration(self, server: str = None) -> dict | str:
        """
        Return the configuration of the server if it is running. Return invalidParameter Exception if not running.

        Parameters
        ----------
        server: str - name of the server to get the configuration for.

        Returns
        -------
        Returns configuration if server is active; InvalidParameter exception thrown otherwise

        """
        if server is None:
            server = self.server_name

        is_known = await self._async_is_server_known(server)
        if is_known is False:
            return "No active configuration found"

        url = self.ops_command_root + "/servers/" + server + "/instance/configuration"

        response = await self._async_make_request("GET", url)
        return response.json().get("omagserverConfig", "No active configuration found")

    def get_active_configuration(self, server: str = None) -> dict | str:
        """
        Return the configuration of the server if it is running. Return invalidParameter Exception if not running.

        Parameters
        ----------
        server: str - name of the server to get the configuration for.

        Returns
        -------
        Returns configuration if server is active; InvalidParameter exception thrown otherwise

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(self._async_get_active_configuration(server))
        return response

#
#   Archive Files
#
    async def _async_add_archive_file(self, archive_file: str, server: str = None, timeout: int = 60) -> None:
        """
        Load the server with the contents of the indicated archive file.

        Parameters
        ----------
        archive_file: the name of the archive file to load
                - note that the path is relative to the working directory of the platform.
        server : Use the server if specified. If None, use the default server associated with the Platform object.
        timeout: Timeout in seconds. Defaults to 30 seconds.
        Returns
        -------
        Response object.  Also throws exceptions if no viable server or endpoint errors

        """
        if server is None:
            server = self.server_name

        url = (
                self.ops_command_root
                + "/servers/" + server
                + "/instance/open-metadata-archives/file"
        )

        await self._async_make_request("POST-DATA", url, archive_file, time_out=timeout)

    def add_archive_file(self, archive_file: str, server: str = None, timeout:int = 30) -> None:
        """
        Load the server with the contents of the indicated archive file.

        Parameters
        ----------
        archive_file: the name of the archive file to load
                - note that the path is relative to the working directory of the platform.
        server : Use the server if specified. If None, use the default server associated with the Platform object.
        timeout: Timeout in seconds. Defaults to 30 seconds.

        Returns
        -------
        Response object.  Also throws exceptions if no viable server or endpoint errors

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self._async_add_archive_file(archive_file, server, timeout))

    async def _async_add_archive(self, archive_connection: str, server: str = None) -> None:
        """ Load the server with the contents of the indicated archive file.

        /open-metadata/server-operations/users/{userId}/server-platform/servers/{serverName}/instance/open-metadata-archives/connection

        Parameters
        ----------
        archive_connection: str
            the name of the archive connection to load from
        server: str
            Use the server if specified. If None, use the default server associated with the Platform object.

        Returns
        -------
        Response object.  Also throws exceptions if no viable server or endpoint errors

        Todo: Need to find a good example of an archive connection to test with
        """
        if server is None:
            server = self.server_name

        url = (
                self.ops_command_root
                + "/servers/" + server
                + "/instance/open-metadata-archives/connection"
        )
        await self._async_make_request("POST-DATA", url, archive_connection)

    def add_archive(self, archive_connection: str, server: str = None) -> None:
        """
        Load the server with the contents of the indicated archive file.

        /open-metadata/server-operations/users/{userId}/server-platform/servers/{serverName}/instance/open-metadata-archives/connection

        Parameters
        ----------
        archive_connection: str
            the name of the archive connection to load from
        server: str
            Use the server if specified. If None, use the default server associated with the Platform object.

        Returns
        -------
        Response object.  Also throws exceptions if no viable server or endpoint errors

        """
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self._async_add_archive_file(archive_connection, server))
#
#   Server Ops
#

    async def _async_get_active_server_status(self, server: str = None) -> dict:
        """  Get the status for the specified server.

        Parameters
        ----------
        server : Use the server if specified. If None, use the default server associated with the Platform object.

        Returns
        -------
        dict: a json object containing the status of the specified server

        Raises
        ------
        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action
        """
        if server is None:
            server = self.server_name

        url = self.admin_command_root + "/servers/" + server + "/instance/status"

        response = await self._async_make_request("GET", url)
        return response

    def get_active_server_status(self, server: str = None) -> dict:
        """
        Get the status for the specified server.
        /open-metadata/platform-services/users/{userId}/server-platform/servers/{server}/instance/status

        Parameters
        ----------
        server : Use the server if specified. If None, use the default server associated with the Platform object.

        Returns
        -------
        dict: a json object containing the status of the specified server

        Raises
        ------
        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action
        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(self._async_get_active_server_status(server))
        return response.json()

    async def _async_get_active_service_list_for_server(self, server: str = None) -> Response:
        """
        List all known active servers on the associated platform.

        /open-metadata/server-operations/users/{userId}/server-platform/servers/{server}/services

        Parameters
        ----------
        server : Use the server if specified. If None, use the default server associated with the Platform object.

        Returns
        -------
        Response object. Also throws exceptions if no viable endpoint or errors

        """
        if server is None:
            server = self.server_name

        url = self.ops_command_root + "/servers/" + server + "/services"
        response = await self._async_make_request("GET", url)
        return response.json().get("serverServicesList")

    def get_active_service_list_for_server(self, server: str = None) -> Response:
        """
        List all known active servers on the associated platform.

        /open-metadata/server-operations/users/{userId}/server-platform/servers/{server}/services

        Parameters
        ----------
        server : Use the server if specified. If None, use the default server associated with the Platform object.

        Returns
        -------
        Response object. Also throws exceptions if no viable endpoint or errors

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(self._async_get_active_service_list_for_server(server))
        return response

#
#   Governance Engine Ops
#
    async def _async_get_governance_engine_summaries(self, server: str = None) -> dict:
        """ Get Governance Engine Summaries. Async version.
        Parameters
        ----------
        server : str, optional
            The name of the server to get governance engine summaries from. If not provided,
            the default server name will be used.

        Returns
        -------
        Response
            The response object containing the governance engine summaries.

        """
        if server is None:
            server = self.server_name

        url = (f"{self.platform_url}/servers/{server}/open-metadata/engine-host/users/{self.user_id}"
               f"/governance-engines/summary")
        response = await self._async_make_request("GET", url)
        return response.json().get("governanceEngineSummaries")

    def get_governance_engine_summaries(self, server: str = None) -> dict:
        """Get Governance Engine Summaries.
        Parameters
        ----------
        server : str, optional
            The name of the server to get governance engine summaries from. If not provided,
            the default server name will be used.

        Returns
        -------
        Response
            The response object containing the governance engine summaries.

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(self._async_get_governance_engine_summaries(server))
        return response

#
# Integration Daemon Ops
#
    async def _async_get_integration_daemon_status(self, server: str = None) -> dict | str:
        """ Get the current status of the integration daemon. Async version.

        """
        if server is None:
            server = self.server_name

        url = f"{self.platform_url}/servers/{server}/open-metadata/integration-daemon/users/{self.user_id}/status"
        response = await self._async_make_request("GET", url)
        return response.json().get("integrationDaemonStatus", "No Integration Groups")
        # return response.json()

    def get_integration_daemon_status(self, server: str = None) -> dict | str:
        """ Get the current status of the integration daemon. Async version.

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(self._async_get_integration_daemon_status(server))
        return response

    async def _async_get_connector_config(self, connector_name: str, server: str = None) -> dict | str:
        """ Retrieve the configuration properties of the named integration connector running in the integration daemon
         - async version"""
        if server is None:
            server = self.server_name
        validate_name(connector_name)

        url = (f"{self.platform_url}/servers/{server}/open-metadata/integration-daemon/users/{self.user_id}/"
               f"integration-connectors/{connector_name}/configuration-properties")

        response = await self._async_make_request("GET", url)
        return response.json()

    def get_connector_config(self, connector_name: str, server: str = None) -> dict | str:
        """ Retrieve the configuration properties of the named integration connector running in the integration
        daemon"""

        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(self._async_get_connector_config(connector_name, server))
        return response

    def get_integration_connector_status(self, server: str = None) -> None:
        """ Get the current status of the integration connector. Async version."""
        self.get_integration_daemon_status(server)
    # todo - finish this? (and do async)
        pass

    async def _async_restart_integration_connector(self, connector_name: str, server: str = None) -> str:
        """ Restart the integration Connector specified by connector_name or all if not specified - async"""

        if server is None:
            server = self.server_name

        url = (f"{self.platform_url}/servers/{server}/open-metadata/integration-daemon/users/"
               f"{self.user_id}/integration-connectors/restart")
        body = {
            "class": "NameRequestBody",
            "name": connector_name
        }
        response = await self._async_make_request("POST", url, body)
        return response

    def restart_integration_connector(self, connector_name: str, server: str = None) -> str:
        """ Restart the integration Connector specified by connector_name or all if not specified"""
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(self._async_restart_integration_connector(connector_name,
                                                                                     server))
        return response

    async def _async_refresh_integration_connectors(self, connector_name: str = 'all', server: str = None,
                                                    time_out: int = 60) -> None:
        """ Issue a refresh request to all connectors running in the integration daemon, or a specific connector
        if one is specified - async version"""
        if server is None:
            server = self.server_name
        if connector_name == 'all':
            connector_name = None

        url = (f"{self.platform_url}/servers/{server}/open-metadata/integration-daemon/users/"
               f"{self.user_id}/integration-connectors/refresh")
        if connector_name:
            body = {
                "class": "NameRequestBody",
                "name": connector_name
            }
            await self._async_make_request("POST", url, body, time_out=time_out
                                           )
        else:
            await self._async_make_request("POST", url, time_out= time_out)

        return

    def refresh_integration_connectors(self, connector_name: str, server: str = None, time_out: int = 60) -> None:
        """ Restart the integration Connector specified by connector_name"""
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self._async_refresh_integration_connectors(connector_name, server, time_out))


if __name__ == "__main__":
    p = ServerOps("meow", "https://127.0.0.1:9443", "garygeeke", verify_flag=False)
    response = p.get_active_service_list_for_server()
    out = response.json()["result"]
    print(out)



#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

Unit tests for the Utils helper functions using the Pytest framework.


A simple display for my profile
"""
import os
import argparse
import time

from rich import box
from rich.console import Console
from rich.table import Table

from pyegeria import (
    InvalidParameterException,
    PropertyServerException,
    UserNotAuthorizedException,
    print_exception_response,
    RegisteredInfo
)

EGERIA_METADATA_STORE = os.environ.get("EGERIA_METADATA_STORE", "active-metadata-store")
EGERIA_KAFKA_ENDPOINT = os.environ.get('KAFKA_ENDPOINT', 'localhost:9092')
EGERIA_PLATFORM_URL = os.environ.get('EGERIA_PLATFORM_URL', 'https://localhost:9443')
EGERIA_VIEW_SERVER = os.environ.get('VIEW_SERVER', 'view-server')
EGERIA_VIEW_SERVER_URL = os.environ.get('EGERIA_VIEW_SERVER_URL', 'https://localhost:9443')
EGERIA_INTEGRATION_DAEMON = os.environ.get('INTEGRATION_DAEMON', 'integration-daemon')
EGERIA_ADMIN_USER = os.environ.get('ADMIN_USER', 'garygeeke')
EGERIA_ADMIN_PASSWORD = os.environ.get('ADMIN_PASSWORD', 'secret')
EGERIA_USER = os.environ.get('EGERIA_USER', 'erinoverview')
EGERIA_USER_PASSWORD = os.environ.get('EGERIA_USER_PASSWORD', 'secret')
EGERIA_JUPYTER = bool(os.environ.get('EGERIA_JUPYTER', 'False'))
EGERIA_WIDTH = int(os.environ.get('EGERIA_WIDTH', '200'))


def display_asset_types(server: str, url: str, username: str, user_password: str,
                        jupyter: bool = EGERIA_JUPYTER, width: int = EGERIA_WIDTH
                        ):
    r_client = RegisteredInfo(server, url, username)
    token = r_client.create_egeria_bearer_token(username, user_password)
    asset_types = r_client.list_asset_types()

    def generate_table() -> Table:
        """Make a new table."""
        table = Table(
            title=f"Asset Types for: {url} @ {time.asctime()}",
            style="bold bright_white on black",
            row_styles=["bold bright_white on black"],
            header_style="white on dark_blue",
            title_style="bold white on black",
            show_lines=True,
            box=box.ROUNDED,
            caption=f"Asset Types from Server '{server}' @ Platform - {url}",
            expand=True
        )

        table.add_column("Name")
        table.add_column("Description")
        table.add_column("SuperType")
        table.add_column("Version")

        name = " "
        description = " "
        version = " "
        super_type = " "
        if len(asset_types) > 0:
            for a_type in asset_types:
                name = a_type.get("name", "none")
                description = a_type.get("description", "none")
                version = a_type.get("version", " ")
                super_type = a_type.get("superType", "none")
                table.add_row(
                    name, description, super_type, str(version)
                )
        return table

    try:
        console = Console(width=width, force_terminal=not jupyter)
        with console.pager(styles=True):
            console.print(generate_table())

    except (InvalidParameterException, PropertyServerException, UserNotAuthorizedException) as e:
        print_exception_response(e)
        assert e.related_http_code != "200", "Invalid parameters"
    finally:
        r_client.close_session()


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--server", help="Name of the server to display status for")
    parser.add_argument("--url", help="URL Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    parser.add_argument("--password", help="User Password")

    args = parser.parse_args()

    server = args.server if args.server is not None else EGERIA_VIEW_SERVER
    url = args.url if args.url is not None else EGERIA_PLATFORM_URL
    userid = args.userid if args.userid is not None else EGERIA_USER
    user_pass = args.password if args.password is not None else EGERIA_USER_PASSWORD
    # guid = args.guid if args.guid is not None else None
    guid = None

    display_asset_types(server, url, userid, user_pass)

if __name__ == "__main__":
    main()


"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.
"""
import inspect
import json
import validators
from json import JSONDecodeError
from pyegeria._exceptions import (
    OMAGCommonErrorCode,
    InvalidParameterException,
)

"""
This package contains internally used validators.

"""


def validate_user_id(user_id: str) -> bool:
    """
    Validate that the provided user id is neither null nor empty.

    Parameters
    ----------
    user_id : str  The user id string to validate

    Returns
    -------
    bool: True if valid, If invalid an InvalidParameterException is raised.

    Raises
    ------
    InvalidParameterException
        If the provided user id is null or empty
    """
    if (user_id is None) or len(user_id) == 0:
        msg = str(OMAGCommonErrorCode.NULL_USER_ID.value["message_template"]).format(
            "user_id"
        )
        calling_frame = inspect.currentframe().f_back
        caller_method = inspect.getframeinfo(calling_frame).function

        exc_msg = json.dumps(
            {
                "class": "VoidResponse",
                "relatedHTTPCode": 400,
                "exceptionClassName": "InvalidParameterException",
                "actionDescription": caller_method,
                "exceptionErrorMessage": msg,
                "exceptionErrorMessageId": OMAGCommonErrorCode.NULL_USER_ID.value[
                    "message_id"
                ],
                "exceptionErrorMessageParameters": user_id,
                "exceptionSystemAction": OMAGCommonErrorCode.NULL_USER_ID.value[
                    "system_action"
                ],
                "exceptionUserAction": OMAGCommonErrorCode.NULL_USER_ID.value[
                    "user_action"
                ],
                "exceptionProperties": {"user_id": user_id},
            }
        )
        raise InvalidParameterException(exc_msg)
    else:
        return True


def validate_server_name(server_name: str) -> bool:
    """
    Validate that the provided server name is neither null nor empty.

    Parameters
    ----------
    server_name : str  The user id string to validate

    Returns
    -------
    bool - True if valid, If invalid an InvalidParameterException is raised.

    """
    calling_frame = inspect.currentframe().f_back
    caller_method = inspect.getframeinfo(calling_frame).function

    if (server_name is None) or (len(server_name) == 0):
        msg = str(
            OMAGCommonErrorCode.SERVER_NAME_NOT_SPECIFIED.value["message_template"]
        )
        exc_msg = json.dumps(
            {
                "class": "VoidResponse",
                "relatedHTTPCode": 400,
                "exceptionClassName": "InvalidParameterException",
                "actionDescription": caller_method,
                "exceptionErrorMessage": msg,
                "exceptionErrorMessageId": OMAGCommonErrorCode.NULL_USER_ID.value[
                    "message_id"
                ],
                "exceptionErrorMessageParameters": server_name,
                "exceptionSystemAction": OMAGCommonErrorCode.NULL_USER_ID.value[
                    "system_action"
                ],
                "exceptionUserAction": OMAGCommonErrorCode.NULL_USER_ID.value[
                    "user_action"
                ],
                "exceptionProperties": {"server_name": server_name},
            }
        )
        raise InvalidParameterException(exc_msg)
    else:
        return True


def validate_guid(guid: str) -> bool:
    """
    Validate that the provided guid is neither null nor empty.

    Parameters
    ----------
    guid : str  The user id string to validate

    Returns
    -------
    bool - True if valid, If invalid an InvalidParameterException is raised.

    """
    calling_frame = inspect.currentframe().f_back
    caller_method = inspect.getframeinfo(calling_frame).function

    if (guid is None) or (len(guid) == 0) or (type(guid) is not str):
        msg = str(OMAGCommonErrorCode.NULL_GUID.value["message_template"])
        exc_msg = json.dumps(
            {
                "class": "VoidResponse",
                "relatedHTTPCode": 400,
                "exceptionClassName": "InvalidParameterException",
                "actionDescription": caller_method,
                "exceptionErrorMessage": msg,
                "exceptionErrorMessageId": OMAGCommonErrorCode.NULL_USER_ID.value[
                    "message_id"
                ],
                "exceptionErrorMessageParameters": guid,
                "exceptionSystemAction": OMAGCommonErrorCode.NULL_USER_ID.value[
                    "system_action"
                ],
                "exceptionUserAction": OMAGCommonErrorCode.NULL_USER_ID.value[
                    "user_action"
                ],
                "exceptionProperties": {"guid": guid},
            }
        )
        raise InvalidParameterException(exc_msg)
    else:
        return True


def validate_name(name: str) -> bool:
    """
    Validate that the provided name is neither null nor empty.

    Parameters
    ----------
    name: str  The user id string to validate

    Returns
    -------
    bool - True if valid, If invalid an InvalidParameterException is raised.

    """
    calling_frame = inspect.currentframe().f_back
    caller_method = inspect.getframeinfo(calling_frame).function

    if (name is None) or (len(name) == 0):
        msg = str(OMAGCommonErrorCode.NULL_NAME.value["message_template"])
        exc_msg = json.dumps(
            {
                "class": "VoidResponse",
                "relatedHTTPCode": 400,
                "exceptionClassName": "InvalidParameterException",
                "actionDescription": caller_method,
                "exceptionErrorMessage": msg,
                "exceptionErrorMessageId": OMAGCommonErrorCode.NULL_USER_ID.value[
                    "message_id"
                ],
                "exceptionErrorMessageParameters": name,
                "exceptionSystemAction": OMAGCommonErrorCode.NULL_USER_ID.value[
                    "system_action"
                ],
                "exceptionUserAction": OMAGCommonErrorCode.NULL_USER_ID.value[
                    "user_action"
                ],
                "exceptionProperties": {"name": name},
            }
        )
        raise InvalidParameterException(exc_msg)
    else:
        return True


def validate_search_string(search_string: str) -> bool:
    """
    Validate that the provided search string is neither null nor empty.

    Parameters
    ----------
    search_string : str  The user id string to validate

    Returns
    -------
    bool - True if valid, If invalid an InvalidParameterException is raised.

    """
    calling_frame = inspect.currentframe().f_back
    caller_method = inspect.getframeinfo(calling_frame).function

    if (search_string is None) or (len(search_string) == 0):
        msg = str(OMAGCommonErrorCode.NULL_SEARCH_STRING.value["message_template"].
                  format('search_string', caller_method))
        exc_msg = json.dumps(
            {
                "class": "VoidResponse",
                "relatedHTTPCode": 400,
                "exceptionClassName": "InvalidParameterException",
                "actionDescription": caller_method,
                "exceptionErrorMessage": msg,
                "exceptionErrorMessageId": OMAGCommonErrorCode.NULL_USER_ID.value[
                    "message_id"
                ],
                "exceptionErrorMessageParameters": [search_string, caller_method],
                "exceptionSystemAction": OMAGCommonErrorCode.NULL_USER_ID.value[
                    "system_action"
                ],
                "exceptionUserAction": OMAGCommonErrorCode.NULL_USER_ID.value[
                    "user_action"
                ],
                "exceptionProperties": {"search_string": search_string},
            }
        )
        raise InvalidParameterException(exc_msg)
    else:
        return True


def validate_public(is_public: bool) -> bool:
    """
    Validate that the provided flag is boolean.

    Parameters
    ----------
    is_public : bool  The flag must be boolean

    Returns
    -------
    bool - True if valid, If invalid an InvalidParameterException is raised.

    """
    calling_frame = inspect.currentframe().f_back
    caller_method = inspect.getframeinfo(calling_frame).function

    if is_public is None:
        msg = str(OMAGCommonErrorCode.NULL_OBJECT.value["message_template"])
        exc_msg = json.dumps(
            {
                "class": "VoidResponse",
                "relatedHTTPCode": 400,
                "exceptionClassName": "InvalidParameterException",
                "actionDescription": caller_method,
                "exceptionErrorMessage": msg,
                "exceptionErrorMessageId": OMAGCommonErrorCode.NULL_USER_ID.value[
                    "message_id"
                ],
                "exceptionErrorMessageParameters": is_public,
                "exceptionSystemAction": OMAGCommonErrorCode.NULL_USER_ID.value[
                    "system_action"
                ],
                "exceptionUserAction": OMAGCommonErrorCode.NULL_USER_ID.value[
                    "user_action"
                ],
                "exceptionProperties": {"is_public": is_public},
            }
        )
        raise InvalidParameterException(exc_msg)
    else:
        return True


def validate_url(url: str) -> bool:
    """
    Validate that the provided url is neither null nor empty. The syntax of the url
    string is also checked to see that it conforms to standards.

    Parameters
    ----------
    url : str  The url string to validate.

    Returns
    -------
    bool - True if valid, If invalid an InvalidParameterException is raised.

    """
    calling_frame = inspect.currentframe().f_back
    caller_method = inspect.getframeinfo(calling_frame).function

    if (url is None) or (len(url) == 0):
        msg = str(
            OMAGCommonErrorCode.SERVER_URL_NOT_SPECIFIED.value["message_template"]
        )
        exc_msg = json.dumps(
            {
                "class": "VoidResponse",
                "relatedHTTPCode": 400,
                "exceptionClassName": "InvalidParameterException",
                "actionDescription": caller_method,
                "exceptionErrorMessage": msg,
                "exceptionErrorMessageId": OMAGCommonErrorCode.SERVER_URL_MALFORMED.value[
                    "message_id"
                ],
                "exceptionErrorMessageParameters": url,
                "exceptionSystemAction": OMAGCommonErrorCode.SERVER_URL_MALFORMED.value[
                    "system_action"
                ],
                "exceptionUserAction": OMAGCommonErrorCode.SERVER_URL_MALFORMED.value[
                    "user_action"
                ],
                "exceptionProperties": {"url": url},
            }
        )
        raise InvalidParameterException(exc_msg)

    # The following hack allows localhost to be used as a hostname - which is disallowed by the
    # validations package
    if ('localhost' in url) and ('localhost.' not in url):
        url = url.replace('localhost', '127.0.0.1')

    result = validators.url(url)
    if result is not True:
        msg = OMAGCommonErrorCode.SERVER_URL_MALFORMED.value["message_template"].format(
            url
        )
        exc_msg = json.dumps(
            {
                "class": "VoidResponse",
                "relatedHTTPCode": 400,
                "exceptionClassName": "InvalidParameterException",
                "actionDescription": caller_method,
                "exceptionErrorMessage": msg,
                "exceptionErrorMessageId": OMAGCommonErrorCode.SERVER_URL_MALFORMED.value[
                    "message_id"
                ],
                "exceptionErrorMessageParameters": url,
                "exceptionSystemAction": OMAGCommonErrorCode.SERVER_URL_MALFORMED.value[
                    "system_action"
                ],
                "exceptionUserAction": OMAGCommonErrorCode.SERVER_URL_MALFORMED.value[
                    "user_action"
                ],
                "exceptionProperties": {"url": url},
            }
        )
        raise InvalidParameterException(exc_msg)
    else:
        return True


def is_json(txt: str) -> bool:
    """
    Parameters
    ----------
    txt : str
        The string to check if it is a valid JSON.

    Returns
    -------
    bool
        True if the string is a valid JSON, False otherwise.
    """
    try:
        json.loads(txt)
        return True
    except (ValueError, JSONDecodeError) as e:
        print(e)
        return False



#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.


A command line interface for Egeria Users - package info

This is an emerging capability based on the **click** package. Feedback welcome!

"""

from .ops_config import Config


#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.


A simple status display for the Integration Daemon.


"""
import argparse
import os
import time

from rich import box
from rich.console import Console
from rich.live import Live
from rich.table import Table

from pyegeria import ServerOps, AutomatedCuration
from pyegeria._exceptions import (
    InvalidParameterException,
    PropertyServerException,
    UserNotAuthorizedException,
    print_exception_response,
)

EGERIA_METADATA_STORE = os.environ.get("EGERIA_METADATA_STORE", "active-metadata-store")
EGERIA_KAFKA_ENDPOINT = os.environ.get("KAFKA_ENDPOINT", "localhost:9092")
EGERIA_PLATFORM_URL = os.environ.get("EGERIA_PLATFORM_URL", "https://localhost:9443")
EGERIA_VIEW_SERVER = os.environ.get("VIEW_SERVER", "view-server")
EGERIA_VIEW_SERVER_URL = os.environ.get(
    "EGERIA_VIEW_SERVER_URL", "https://localhost:9443"
)
EGERIA_INTEGRATION_DAEMON = os.environ.get("INTEGRATION_DAEMON", "integration-daemon")
EGERIA_INTEGRATION_DAEMON_URL = os.environ.get(
    "EGERIA_INTEGRATION_DAEMON_URL", "https://localhost:9443"
)
EGERIA_ADMIN_USER = os.environ.get("ADMIN_USER", "garygeeke")
EGERIA_ADMIN_PASSWORD = os.environ.get("ADMIN_PASSWORD", "secret")
EGERIA_USER = os.environ.get("EGERIA_USER", "erinoverview")
EGERIA_USER_PASSWORD = os.environ.get("EGERIA_USER_PASSWORD", "secret")
EGERIA_JUPYTER = bool(os.environ.get("EGERIA_JUPYTER", "False"))
EGERIA_WIDTH = int(os.environ.get("EGERIA_WIDTH", "200"))

disable_ssl_warnings = True


def display_integration_daemon_status(
    integ_server: str,
    integ_url: str,
    view_server: str,
    view_url: str,
    user: str,
    user_pass: str,
    paging: bool,
    jupyter: bool = EGERIA_JUPYTER,
    width: int = EGERIA_WIDTH,
    sort: bool = True,
):
    s_client = ServerOps(integ_server, integ_url, user)
    a_client = AutomatedCuration(view_server, view_url, user, user_pass)
    token = a_client.create_egeria_bearer_token()

    def generate_table() -> Table:
        """Make a new table."""
        table = Table(
            title=f"Integration Daemon Status @ {time.asctime()}",
            style="bold white on black",
            row_styles=["bold white on black"],
            header_style="white on dark_blue",
            title_style="bold white on black",
            caption_style="white on black",
            show_lines=True,
            box=box.ROUNDED,
            caption=f"Integration Daemon Status for Server '{integ_server}' @ Platform - {integ_url}",
            expand=True,
        )
        table.add_column("Connector Name", min_width=15)
        table.add_column("Status", max_width=6)

        table.add_column("Last Refresh Time", min_width=12)
        table.add_column("Min Refresh (mins)", max_width=6)
        table.add_column("Target Element", min_width=20)
        table.add_column("Exception Message", min_width=10)

        daemon_status = s_client.get_integration_daemon_status()

        reports = daemon_status["integrationConnectorReports"]
        if sort is True:
            connector_reports = sorted(
                reports, key=lambda x: x.get("connectorName", "---")
            )
        else:
            connector_reports = reports

        for connector in connector_reports:
            connector_name = connector.get("connectorName", "---")
            connector_status = connector.get("connectorStatus", "---")
            connector_guid = connector.get("connectorGUID", "---")
            last_refresh_time = connector.get("lastRefreshTime", "---")[:-10]
            refresh_interval = str(connector.get("minMinutesBetweenRefresh", "---"))
            exception_msg = " "
            if connector_guid != "---":
                targets = a_client.get_catalog_targets(connector_guid)
                tgt_tab = Table()
                tgt_tab.add_column("Target")
                tgt_tab.add_column("UniqueName")
                tgt_tab.add_column("Relationship GUID", no_wrap=True)

                if type(targets) == list:
                    targets_md = True
                    for target in targets:
                        t_name = target["catalogTargetName"]
                        # t_sync = target["permittedSynchronization"]
                        t_unique_name = target["catalogTargetElement"]["uniqueName"]
                        t_rel_guid = target["relationshipGUID"]
                        # targets_m += f"* Target Name: __{t_name}__\n* Sync: {t_sync}\n* Unique Name: {t_unique_name}\n\n"
                        tgt_tab.add_row(t_name, t_unique_name, t_rel_guid)
                    # targets_md = Markdown(targets_m)
                else:
                    targets_md = False
            else:
                targets_md = False
            if targets_md is False:
                targets_out = ""
            else:
                targets_out = tgt_tab

            if connector_status in ("RUNNING", "REFRESHING", "WAITING"):
                connector_status = f"[green]{connector_status}"
            elif connector_status in ("INITIALIZE FAILED", "CONFIG_FAILED", "FAILED"):
                connector_status = f"[red]{connector_status}"
            else:
                connector_status = f"[yellow]{connector_status}"

            table.add_row(
                connector_name,
                connector_status,
                last_refresh_time,
                refresh_interval,
                targets_out,
                exception_msg,
            )
        return table

    try:
        if paging is True:
            console = Console(width=width, force_terminal=not jupyter)
            with console.pager():
                console.print(generate_table())
        else:
            with Live(
                generate_table(),
                refresh_per_second=1,
                screen=True,
                vertical_overflow="visible",
            ) as live:
                while True:
                    time.sleep(2)
                    live.update(generate_table())

    except (
        InvalidParameterException,
        PropertyServerException,
        UserNotAuthorizedException,
    ) as e:
        print_exception_response(e)

    except KeyboardInterrupt:
        pass

    finally:
        s_client.close_session()
        a_client.close_session()


def main_live():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--integ_server", help="Name of the integration server to display status for"
    )
    parser.add_argument("--integ_url", help="URL Platform to connect to")
    parser.add_argument("--view_server", help="Name of the view server to use")
    parser.add_argument("--view_url", help="view server URL Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    parser.add_argument("--password", help="User Password")
    args = parser.parse_args()

    integ_server = (
        args.integ_server
        if args.integ_server is not None
        else EGERIA_INTEGRATION_DAEMON
    )
    integ_url = (
        args.integ_url if args.integ_url is not None else EGERIA_INTEGRATION_DAEMON_URL
    )
    view_server = (
        args.view_server if args.view_server is not None else EGERIA_VIEW_SERVER
    )
    view_url = args.view_url if args.view_url is not None else EGERIA_VIEW_SERVER_URL
    userid = args.userid if args.userid is not None else EGERIA_USER
    user_pass = args.password if args.password is not None else EGERIA_USER_PASSWORD
    display_integration_daemon_status(
        integ_server=integ_server,
        integ_url=integ_url,
        view_server=view_server,
        view_url=view_url,
        user=userid,
        user_pass=user_pass,
        paging=False,
    )


def main_paging():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--integ_server", help="Name of the integration server to display status for"
    )
    parser.add_argument("--integ_url", help="URL Platform to connect to")
    parser.add_argument("--view_server", help="Name of the view server to use")
    parser.add_argument("--view_url", help="view server URL Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    parser.add_argument("--password", help="User Password")
    args = parser.parse_args()

    integ_server = (
        args.integ_server
        if args.integ_server is not None
        else EGERIA_INTEGRATION_DAEMON
    )
    integ_url = (
        args.integ_url if args.integ_url is not None else EGERIA_INTEGRATION_DAEMON_URL
    )
    view_server = (
        args.view_server if args.view_server is not None else EGERIA_VIEW_SERVER
    )
    view_url = args.view_url if args.view_url is not None else EGERIA_VIEW_SERVER_URL
    userid = args.userid if args.userid is not None else EGERIA_USER
    user_pass = args.password if args.password is not None else EGERIA_USER_PASSWORD
    display_integration_daemon_status(
        integ_server=integ_server,
        integ_url=integ_url,
        view_server=view_server,
        view_url=view_url,
        user=userid,
        user_pass=user_pass,
        paging=True,
    )


if __name__ == "__main__":
    main_live()

if __name__ == "__main_paging__":
    main_paging()



#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

Display the status of cataloged platforms and servers.
"""
import json
import os
import time
import argparse
from datetime import datetime
from rich.prompt import Prompt
from rich.table import Table
from rich.live import Live
from rich.console import Console
from rich.markdown import Markdown
from confluent_kafka import Consumer, KafkaException

EGERIA_METADATA_STORE = os.environ.get("EGERIA_METADATA_STORE", "active-metadata-store")
EGERIA_KAFKA_ENDPOINT = os.environ.get('KAFKA_ENDPOINT', 'localhost:9192')
EGERIA_PLATFORM_URL = os.environ.get('EGERIA_PLATFORM_URL', 'https://localhost:9443')
EGERIA_VIEW_SERVER = os.environ.get('VIEW_SERVER', 'view-server')
EGERIA_VIEW_SERVER_URL = os.environ.get('EGERIA_VIEW_SERVER_URL', 'https://localhost:9443')
EGERIA_INTEGRATION_DAEMON = os.environ.get('INTEGRATION_DAEMON', 'integration-daemon')
EGERIA_ADMIN_USER = os.environ.get('ADMIN_USER', 'garygeeke')
EGERIA_ADMIN_PASSWORD = os.environ.get('ADMIN_PASSWORD', 'secret')
EGERIA_USER = os.environ.get('EGERIA_USER', 'erinoverview')
EGERIA_USER_PASSWORD = os.environ.get('EGERIA_USER_PASSWORD', 'secret')
EGERIA_JUPYTER = os.environ.get('EGERIA_JUPYTER', False)
EGERIA_WIDTH = os.environ.get('EGERIA_WIDTH', 200)

def main(ep: str = EGERIA_KAFKA_ENDPOINT):

    disable_ssl_warnings = True
    console = Console(width=200)
    now = datetime.now()
    current_time = now.strftime("%H:%M:%S")

    earliest_latest = Prompt.ask("Msgs from earliest or latest:", default="earliest")

    # Define the Kafka consumer configuration.
    config = {
        'bootstrap.servers': ep,  # replace with your Kafka broker(s)
        'group.id': f"view_asset_events:{current_time}",  # replace with your consumer group
        'auto.offset.reset': earliest_latest  # can be set to 'earliest' or 'latest'
    }
    print(f"Kafka config is {json.dumps(config)}")
    # Initialize a Kafka consumer.
    consumer = Consumer(config)

    # Subscribe to a Kafka topic.
    consumer.subscribe(['egeria.omag.server.active-metadata-store.omas.assetconsumer.outTopic'])  # replace with your Kafka topic

    try:
        while True:
            msg = consumer.poll(2.0)  # timeout set to 1 second

            if msg is None:
                continue
            elif msg.error():
                raise KafkaException(msg.error())
            else:
                event = json.loads(msg.value())
                event_time = event["eventTime"]
                event_type = event["eventType"]
                guid = event["elementHeader"]["guid"]

                type_name = event["elementHeader"]["type"]["typeName"]
                origin = event["elementHeader"]["origin"]["sourceServer"]

                element_properties = event["elementProperties"]
                element_properties_keys = element_properties.keys()
                props = " "
                for key in element_properties_keys:
                    value = element_properties[key]
                    props += f"* {key}: {value}\n"
                console.rule(style= "[bold red]")
                console.rule(f"\tMessage TimeStamp: {event_time}\t eventType: {event_type}\t typeName: {type_name}\t guid: {guid}")
                msg = (

                          f"properties: \n[bright white on black]{props}\n\n")
                msg = Markdown(msg)

                console.print(msg)

    except KeyboardInterrupt:
        pass

    finally:
        # Close down consumer to commit final offsets.
        consumer.close()

if __name__ == "__main__":
    parser = argparse.ArgumentParser()

    parser.add_argument("--ep", help="Endpoint to connect to")
    args = parser.parse_args()

    ep = args.ep if args.ep is not None else EGERIA_KAFKA_ENDPOINT

    main(ep)


"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

This is the pyegeria client package. The purpose of the package is to provide
easy access to Egeria (https://egeria-project.org). The package is currently in
development. 

The first capabilities are around Egeria's platform services used to start and stop
the server platform and servers. 

"""

from ._globals import (
    is_debug,
    disable_ssl_warnings,
    max_paging_size,
    TEMPLATE_GUIDS,
    INTEGRATION_GUIDS,
    default_time_out,
)

if disable_ssl_warnings:
    from urllib3.exceptions import InsecureRequestWarning
    from urllib3 import disable_warnings

    disable_warnings(InsecureRequestWarning)

from ._exceptions import (
    InvalidParameterException,
    PropertyServerException,
    UserNotAuthorizedException,
    print_exception_response,
)

# from .utils import print_response, body_slimmer, wrap_text
from .utils import print_response, body_slimmer
from ._client import Client
from .automated_curation_omvs import AutomatedCuration
from .core_omag_server_config import CoreServerConfig
from .platform_services import Platform
from .registered_info import RegisteredInfo
from .glossary_browser_omvs import GlossaryBrowser
from ._validators import (
    validate_user_id,
    validate_name,
    validate_guid,
    validate_server_name,
    validate_search_string,
    validate_url,
    is_json,
    validate_public,
)
from .asset_catalog_omvs import AssetCatalog

from .my_profile_omvs import MyProfile
from .full_omag_server_config import FullServerConfig
from .server_operations import ServerOps
from .collection_manager_omvs import CollectionManager
from .project_manager_omvs import ProjectManager
from .valid_metadata_omvs import ValidMetadataManager
from .asset_catalog_omvs import AssetCatalog
from .Xloaded_resources_omvs import LoadedResources
from ._deprecated_gov_engine import GovEng
from .runtime_manager_omvs import RuntimeManager
from .action_author_omvs import GovernanceAuthor
from .glossary_manager_omvs import GlossaryManager
from .create_tech_guid_lists import build_global_guid_lists
from .classification_manager_omvs import ClassificationManager
from .feedback_manager_omvs import FeedbackManager
from .mermaid_utilities import load_mermaid, render_mermaid, generate_process_graph


#
# The following assignments were generated by the `create_tech_guid_lists.py` utility that uses the pyegeria functions
# to interrogate Egeria (and the Core Content Pack) for the GUIDS associates with integration connectors and templates.
# While these GUIDS rarely change, they will evolve as new connectors are built and integrated. This could impact some
# of the pyegeria methods that, as a convenience, use pre-assigned GUIDS to create new elements and such.
#
# This list may also be augmented (and of use) to those wishing to extend the pyegeria methods to automate or simplify
# the use of custom connectors and templates.
#
#
global TEMPLATE_GUIDS, INTEGRATION_GUIDS

TEMPLATE_GUIDS["CSV Data File"] = "13770f93-13c8-42be-9bb8-e0b1b1e52b1f"
TEMPLATE_GUIDS["Keystore File"] = "fbcfcc0c-1652-421f-b49b-c3e1c108768f"
TEMPLATE_GUIDS["Unity Catalog Server"] = "dcca9788-b30f-4007-b1ac-ec634aff6879"
INTEGRATION_GUIDS["UnityCatalogInsideCatalog"] = "74dde22f-2249-4ea3-af2b-b39e73f79b81"
INTEGRATION_GUIDS["UnityCatalogServer"] = "06d068d9-9e08-4e67-8c59-073bbf1013af"
INTEGRATION_GUIDS["JDBC"] = "70dcd0b7-9f06-48ad-ad44-ae4d7a7762aa"
TEMPLATE_GUIDS["Data File"] = "66d8dda9-00cf-4e59-938c-4b0583596b1e"
TEMPLATE_GUIDS["View Server"] = "fd61ca01-390d-4aa2-a55d-426826aa4e1b"
TEMPLATE_GUIDS["Archive File"] = "7578e504-d00f-406d-a194-3fc0a351cdf9"
TEMPLATE_GUIDS["Unity Catalog Catalog"] = "5ee006aa-a6d6-411b-9b8d-5f720c079cae"
TEMPLATE_GUIDS[
    "PostgreSQL Relational Database"
] = "3d398b3f-7ae6-4713-952a-409f3dea8520"
INTEGRATION_GUIDS["JDBC"] = "70dcd0b7-9f06-48ad-ad44-ae4d7a7762aa"
TEMPLATE_GUIDS["Program File"] = "32d27e9c-1fdf-455a-ad2a-42b4d7d99108"
TEMPLATE_GUIDS["FileFolder"] = "fbdd8efd-1b69-474c-bb6d-0a304b394146"
INTEGRATION_GUIDS["ContentPacksMonitor"] = "6bb2181e-7724-4515-ba3c-877cded55980"
INTEGRATION_GUIDS["GeneralFilesMonitor"] = "1b98cdac-dd0a-4621-93db-99ef5a1098bc"
INTEGRATION_GUIDS["SampleDataFilesMonitor"] = "cd6479e1-2fe7-4426-b358-8a0cf70be117"
INTEGRATION_GUIDS["OpenLineageFilePublisher"] = "6271b678-7d22-4cdf-87b1-45b366beaf4e"
TEMPLATE_GUIDS["PostgreSQL Server"] = "542134e6-b9ce-4dce-8aef-22e8daf34fdb"
INTEGRATION_GUIDS["PostgreSQLServer"] = "36f69fd0-54ba-4f59-8a44-11ccf2687a34"
TEMPLATE_GUIDS["Audio Data File"] = "39b4b670-7f15-4744-a5ba-62e8edafbcee"
TEMPLATE_GUIDS["Document File"] = "eb6f728d-fa54-4350-9807-1199cbf96851"
TEMPLATE_GUIDS["Integration Daemon"] = "6b3516f0-dd13-4786-9601-07215f995197"
TEMPLATE_GUIDS["XML Data File"] = "ea67ae71-c674-473e-b38b-689879d2a7d9"
TEMPLATE_GUIDS["Unity Catalog Schema"] = "5bf92b0f-3970-41ea-b0a3-aacfbf6fd92e"
TEMPLATE_GUIDS["Parquet Data File"] = "7f6cd744-79c3-4d25-a056-eeb1a91574c3"
TEMPLATE_GUIDS["3D Image Data File"] = "0059ea2b-6292-4cac-aa6f-a80a605f1114"
TEMPLATE_GUIDS["YAML File"] = "2221855b-2b64-4b45-a2ee-c40adc5e2a64"
TEMPLATE_GUIDS["Metadata Access Server"] = "bd8de890-fa79-4c24-aab8-20b41b5893dd"
TEMPLATE_GUIDS["Properties File"] = "3b281111-a0ef-4fc4-99e7-9a0ef84a7636"
TEMPLATE_GUIDS["Vector Data File"] = "db1bec7f-55a9-40d3-91c0-a57b76d422e2"
TEMPLATE_GUIDS["Apache Kafka Server"] = "5e1ff810-5418-43f7-b7c4-e6e062f9aff7"
INTEGRATION_GUIDS["KafkaTopic"] = "fa1f711c-0b34-4b57-8e6e-16162b132b0c"
TEMPLATE_GUIDS["Executable File"] = "3d99a163-7a13-4576-a212-784010a8302a"
INTEGRATION_GUIDS["OpenLineageAPIPublisher"] = "2156bc98-973a-4859-908d-4ccc96f53cc5"
TEMPLATE_GUIDS["Unity Catalog Table"] = "6cc1e5f5-4c1e-4290-a80e-e06643ffb13d"
TEMPLATE_GUIDS["JSON Data File"] = "c4836635-7e9e-446a-83b5-15e206b1aff3"
TEMPLATE_GUIDS["File System"] = "522f228c-097c-4f90-9efc-26c1f2696f87"
TEMPLATE_GUIDS["Source Code File"] = "9c7013ef-f29b-4b01-a8ea-5ea14f64c67a"
TEMPLATE_GUIDS["Apple MacBook Pro"] = "32a9fd56-85c9-47fe-a211-9da3871bf4da"
TEMPLATE_GUIDS["Build Instruction File"] = "fbb2fa2e-8bcb-402e-9be7-5c6db9f2c504"
TEMPLATE_GUIDS["Spreadsheet Data File"] = "e4fabff5-2ba9-4050-9076-6ed917970b4c"
TEMPLATE_GUIDS["UNIX File System"] = "27117270-8667-41d0-a99a-9118f9b60199"
TEMPLATE_GUIDS["Video Data File"] = "93b2b722-ec0f-4da4-960a-b8d4922f8bf5"
TEMPLATE_GUIDS["Unity Catalog Function"] = "a490ba65-6104-4213-9be9-524e16fed8aa"
TEMPLATE_GUIDS[
    "PostgreSQL Relational Database Schema"
] = "82a5417c-d882-4271-8444-4c6a996a8bfc"
TEMPLATE_GUIDS["Engine Host"] = "1764a891-4234-45f1-8cc3-536af40c790d"
TEMPLATE_GUIDS["Avro Data File"] = "9f5be428-058e-41dd-b506-3a222283b579"
TEMPLATE_GUIDS["Unity Catalog Volume"] = "92d2d2dc-0798-41f0-9512-b10548d312b7"
TEMPLATE_GUIDS["File"] = "ae3067c7-cc72-4a18-88e1-746803c2c86f"
TEMPLATE_GUIDS["Apache Kafka Topic"] = "ea8f81c9-c59c-47de-9525-7cc59d1251e5"
TEMPLATE_GUIDS["Script File"] = "dbd5e6bb-1ff8-46f4-a007-fb0485f68c92"
TEMPLATE_GUIDS["Apache Atlas Server"] = "fe6dce45-a978-4417-ab55-17f05b8bcea7"
TEMPLATE_GUIDS["Raster Data File"] = "47211156-f03f-4881-8526-015e695a3dac"
TEMPLATE_GUIDS["Data Folder"] = "372a0379-7060-4c9d-8d84-bc709b31794c"
INTEGRATION_GUIDS[
    "MaintainDataFolderLastUpdateDate"
] = "fd26f07c-ae44-4bc5-b457-37b43112224f"
INTEGRATION_GUIDS["OpenLineageFilePublisher"] = "6271b678-7d22-4cdf-87b1-45b366beaf4e"
TEMPLATE_GUIDS["OMAG Server Platform"] = "9b06c4dc-ddc8-47ae-b56b-28775d3a96f0"
INTEGRATION_GUIDS["OpenAPI"] = "b89d9a5a-2ea6-49bc-a4fc-e7df9f3ca93e"
INTEGRATION_GUIDS["OMAGServerPlatform"] = "dee84e6e-7a96-4975-86c1-152fb3ab759b"
INTEGRATION_GUIDS["PostgreSQLServerCataloguer"] = "36f69fd0-54ba-4f59-8a44-11ccf2687a34"
INTEGRATION_GUIDS[
    "UnityCatalogInsideCatalogSynchronizer"
] = "74dde22f-2249-4ea3-af2b-b39e73f79b81"
INTEGRATION_GUIDS["OpenLineageAPIPublisher"] = "2156bc98-973a-4859-908d-4ccc96f53cc5"
INTEGRATION_GUIDS["JDBCDatabaseCataloguer"] = "70dcd0b7-9f06-48ad-ad44-ae4d7a7762aa"
INTEGRATION_GUIDS["FilesCataloguer"] = "1b98cdac-dd0a-4621-93db-99ef5a1098bc"
INTEGRATION_GUIDS["SampleDataCataloguer"] = "cd6479e1-2fe7-4426-b358-8a0cf70be117"
INTEGRATION_GUIDS[
    "OpenLineageGovernanceActionPublisher"
] = "206f8faf-04da-4b6f-8280-eeee3943afeb"
INTEGRATION_GUIDS["OpenAPICataloguer"] = "b89d9a5a-2ea6-49bc-a4fc-e7df9f3ca93e"
INTEGRATION_GUIDS[
    "OMAGServerPlatformCataloguer"
] = "dee84e6e-7a96-4975-86c1-152fb3ab759b"
INTEGRATION_GUIDS["ApacheKafkaCataloguer"] = "fa1f711c-0b34-4b57-8e6e-16162b132b0c"
INTEGRATION_GUIDS[
    "UnityCatalogServerSynchronizer"
] = "06d068d9-9e08-4e67-8c59-073bbf1013af"
INTEGRATION_GUIDS["MaintainLastUpdateDate"] = "fd26f07c-ae44-4bc5-b457-37b43112224f"
INTEGRATION_GUIDS["OpenLineageKafkaListener"] = "980b989c-de78-4e6a-a58d-51049d7381bf"
INTEGRATION_GUIDS["OpenLineageFilePublisher"] = "6271b678-7d22-4cdf-87b1-45b366beaf4e"
INTEGRATION_GUIDS["ContentPacksCataloguer"] = "6bb2181e-7724-4515-ba3c-877cded55980"
INTEGRATION_GUIDS["OpenLineageCataloguer"] = "3347ac71-8dd2-403a-bc16-75a71be64bd7"



"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.



Execute ToDo actions.

"""
import os
import time
from datetime import datetime

import click

# from ops_config import Config, pass_config
from pyegeria import MyProfile
from pyegeria._exceptions import (
    InvalidParameterException,
    PropertyServerException,
    print_exception_response,
)

erins_guid = "a588fb08-ae09-4415-bd5d-991882ceacba"
peter_guid = "a187bc48-8154-491f-97b4-a2f3c3f1a00e"
tanya_guid = "26ec1614-bede-4b25-a2a3-f8ed26db3aaa"

ERIA_METADATA_STORE = os.environ.get("EGERIA_METADATA_STORE", "active-metadata-store")
EGERIA_KAFKA_ENDPOINT = os.environ.get('KAFKA_ENDPOINT', 'localhost:9092')
EGERIA_PLATFORM_URL = os.environ.get('EGERIA_PLATFORM_URL', 'https://localhost:9443')
EGERIA_VIEW_SERVER = os.environ.get('VIEW_SERVER', 'view-server')
EGERIA_VIEW_SERVER_URL = os.environ.get('EGERIA_VIEW_SERVER_URL', 'https://localhost:9443')
EGERIA_INTEGRATION_DAEMON = os.environ.get('INTEGRATION_DAEMON', 'integration-daemon')
EGERIA_INTEGRATION_DAEMON_URL = os.environ.get('EGERIA_INTEGRATION_DAEMON_URL', 'https://localhost:9443')
EGERIA_ADMIN_USER = os.environ.get('ADMIN_USER', 'garygeeke')
EGERIA_ADMIN_PASSWORD = os.environ.get('ADMIN_PASSWORD', 'secret')
EGERIA_USER = os.environ.get('EGERIA_USER', 'erinoverview')
EGERIA_USER_PASSWORD = os.environ.get('EGERIA_USER_PASSWORD', 'secret')


@click.command('create-todo')
@click.option('--server', default=EGERIA_VIEW_SERVER, help='Egeria metadata store to load')
@click.option('--url', default=EGERIA_VIEW_SERVER_URL, help='URL of Egeria platform to connect to')
@click.option('--userid', default=EGERIA_USER, help='Egeria user')
@click.option('--password', default=EGERIA_USER_PASSWORD, help='Egeria user password')
@click.option('--timeout', default=60, help='Number of seconds to wait')
@click.option('--name', prompt='Todo Name', help='Name of Todo', required=True)
@click.option('--description', prompt='Description', help='Brief description of To Do item', required=True)
@click.option('--type', prompt='Todo Type', help='Type of Todo', required=True, default='forMe')
@click.option('--priority', prompt='Todo Priority', type=int, help='Priority of Todo', required=True, default=0)
@click.option('--due', prompt='Due Date', help='Due date of Todo (yyyy-mm-dd)',
              default = datetime.now().strftime("%Y-%m-%d"), required=True)
@click.option('--assigned-to', prompt='Assigned to', help='Party the Todo is assigned to', required=True,
              default=peter_guid)
def create_todo(server, url, userid, password, timeout, name, description, type, priority, due, assigned_to):
    """Create a new ToDo item"""
    m_client = MyProfile(server, url, user_id=userid, user_pwd=password)
    token = m_client.create_egeria_bearer_token()
    try:
        body = {
            "properties": {
                "class": "ToDoProperties",
                "qualifiedName": f"{name}-{time.asctime()}",
                "name": name,
                "description": description,
                "toDoType": type,
                "priority": priority,
                "dueTime": due,
                "status": "OPEN"
            },
            "assignToActorGUID": assigned_to
        }

        resp = m_client.create_to_do(body)
        # if type(resp) is str:
        click.echo(f"Response was {resp}")
        # elif type(resp) is dict:
        #     click.echo(json.dumps(resp), indent = 2)

    except (InvalidParameterException, PropertyServerException) as e:
        print_exception_response(e)
    finally:
        m_client.close_session()


@click.command('delete-todo')
@click.option('--server', default=EGERIA_VIEW_SERVER, help='Egeria metadata store to load')
@click.option('--url', default=EGERIA_VIEW_SERVER_URL, help='URL of Egeria platform to connect to')
@click.option('--userid', default=EGERIA_USER, help='Egeria user')
@click.option('--password', default=EGERIA_USER_PASSWORD, help='Egeria user password')
@click.option('--timeout', default=60, help='Number of seconds to wait')
@click.argument('todo-guid')
def delete_todo(server, url, userid, password, timeout, todo_guid):
    """Delete the todo item specified """
    m_client = MyProfile(server, url, user_id=userid, user_pwd=password)
    token = m_client.create_egeria_bearer_token()
    try:
        m_client.delete_to_do(todo_guid)

        click.echo(f"Deleted Todo item {todo_guid}")

    except (InvalidParameterException, PropertyServerException) as e:
        print_exception_response(e)
    finally:
        m_client.close_session()


@click.command('change-todo-status')
@click.argument('todo-guid')
@click.option('--server', default=EGERIA_VIEW_SERVER, help='Egeria metadata store to load')
@click.option('--url', default=EGERIA_VIEW_SERVER_URL, help='URL of Egeria platform to connect to')
@click.option('--userid', default=EGERIA_USER, help='Egeria user')
@click.option('--password', default=EGERIA_USER_PASSWORD, help='Egeria user password')
@click.option('--timeout', default=60, help='Number of seconds to wait')
@click.option('--new-status', type=click.Choice(['OPEN', 'IN_PROGRESS', 'WAITING', 'COMPLETE', 'ABANDONED'],
                                                case_sensitive='False'), help='Enter the new ToDo item status',
              required=True)
def change_todo_status(server, url, userid, password, timeout, todo_guid, new_status):
    """Update a ToDo item status"""
    m_client = MyProfile(server, url, user_id=userid, user_pwd=password)
    token = m_client.create_egeria_bearer_token()
    try:

        body = {
                "class": "ToDoProperties",
                "toDoStatus": new_status
        }

        m_client.update_to_do(todo_guid, body, is_merge_update=True)

        click.echo(f"Changed todo item {todo_guid} status to {new_status}.")

    except (InvalidParameterException, PropertyServerException) as e:
        print_exception_response(e)
    finally:
        m_client.close_session()


@click.command('mark-todo-complete')
@click.option('--server', default=EGERIA_VIEW_SERVER, help='Egeria metadata store to load')
@click.option('--url', default=EGERIA_VIEW_SERVER_URL, help='URL of Egeria platform to connect to')
@click.option('--userid', default=EGERIA_USER, help='Egeria user')
@click.option('--password', default=EGERIA_USER_PASSWORD, help='Egeria user password')
@click.option('--timeout', default=60, help='Number of seconds to wait')
@click.argument('todo-guid')
def mark_todo_complete(server, url, userid, password, timeout, todo_guid):
    """Mark the specified todo as complete"""
    m_client = MyProfile(server, url, user_id=userid, user_pwd=password)
    try:
        token = m_client.create_egeria_bearer_token()
        body = {
                "class": "ToDoProperties",
                "completionTime": time.asctime(),
                "toDoStatus": "COMPLETE"
        }

        m_client.update_to_do(todo_guid, body, is_merge_update=True)

        click.echo(f"Marked todo item {todo_guid} as complete.")

    except (InvalidParameterException, PropertyServerException) as e:
        print_exception_response(e)
    finally:
        m_client.close_session()


@click.command('reassign-todo')
@click.option('--server', default=EGERIA_VIEW_SERVER, help='Egeria metadata store to load')
@click.option('--url', default=EGERIA_VIEW_SERVER_URL, help='URL of Egeria platform to connect to')
@click.option('--userid', default=EGERIA_USER, help='Egeria user')
@click.option('--password', default=EGERIA_USER_PASSWORD, help='Egeria user password')
@click.option('--timeout', default=60, help='Number of seconds to wait')
@click.argument('todo-guid')
@click.argument('new-actor-guid')
def reassign_todo(server, url, userid, password, timeout, todo_guid, new_actor_guid):
    """Reassign ToDo item to new actor"""
    m_client = MyProfile(server, url, user_id=userid, user_pwd=password)
    token = m_client.create_egeria_bearer_token()
    try:

        m_client.reassign_to_do(todo_guid, new_actor_guid)

        click.echo(f"Reassigned Todo item {todo_guid} to {new_actor_guid}")

    except (InvalidParameterException, PropertyServerException) as e:
        print_exception_response(e)
    finally:
        m_client.close_session()



#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

Unit tests for the Utils helper functions using the Pytest framework.


A simple status display for Governance Actions
"""
import argparse
import json
import os
import time

from rich import box
from rich.console import Console
from rich.live import Live
from rich.table import Table

from pyegeria import (
    InvalidParameterException,
    PropertyServerException,
    UserNotAuthorizedException,
    print_exception_response,
)
from pyegeria.server_operations import ServerOps

EGERIA_METADATA_STORE = os.environ.get("EGERIA_METADATA_STORE", "active-metadata-store")
EGERIA_KAFKA_ENDPOINT = os.environ.get("KAFKA_ENDPOINT", "localhost:9092")
EGERIA_PLATFORM_URL = os.environ.get("EGERIA_PLATFORM_URL", "https://localhost:9443")
EGERIA_VIEW_SERVER = os.environ.get("VIEW_SERVER", "view-server")
EGERIA_VIEW_SERVER_URL = os.environ.get(
    "EGERIA_VIEW_SERVER_URL", "https://localhost:9443"
)
EGERIA_INTEGRATION_DAEMON = os.environ.get("INTEGRATION_DAEMON", "integration-daemon")
EGERIA_ENGINE_HOST = os.environ.get("INTEGRATION_ENGINE_HOST", "engine-host")
EGERIA_ENGINE_HOST_URL = os.environ.get(
    "INTEGRATION_ENGINE_HOST_URL", "https://localhost:9443"
)
EGERIA_ADMIN_USER = os.environ.get("ADMIN_USER", "garygeeke")
EGERIA_ADMIN_PASSWORD = os.environ.get("ADMIN_PASSWORD", "secret")
EGERIA_USER = os.environ.get("EGERIA_USER", "erinoverview")
EGERIA_USER_PASSWORD = os.environ.get("EGERIA_USER_PASSWORD", "secret")
EGERIA_JUPYTER = bool(os.environ.get("EGERIA_JUPYTER", "False"))
EGERIA_WIDTH = int(os.environ.get("EGERIA_WIDTH", "200"))


disable_ssl_warnings = True


def display_gov_eng_status(
    server: str,
    url: str,
    username: str,
    user_pass: str,
    paging: bool,
    jupyter: bool = EGERIA_JUPYTER,
    width: int = EGERIA_WIDTH,
):
    server_name = server
    s_client = ServerOps(server_name, url, username, user_pass)

    def generate_table() -> Table:
        """Make a new table."""
        table = Table(
            title=f"Governance Engine Status @ {time.asctime()}",
            style="bold white on black",
            row_styles=["bold white on black"],
            header_style="white on dark_blue",
            title_style="bold white on black",
            caption_style="white on black",
            show_lines=True,
            box=box.ROUNDED,
            caption=f"Server: '{server_name}' running on {url}",
            expand=True,
        )
        # table.footer: f"Server {server_name} on Platform {good_platform1_url}"
        table.add_column("Gov Engine")
        table.add_column("Gov Engine Type")
        table.add_column("Gov Engine Desc")
        table.add_column("Engine Status")
        table.add_column("Request Types")

        gov_eng_status = s_client.get_governance_engine_summaries()
        sorted_gov_eng_status = sorted(
            gov_eng_status, key=lambda k: k["governanceEngineName"]
        )
        for engine in sorted_gov_eng_status:
            gov_eng = engine["governanceEngineName"]
            eng_type = engine["governanceEngineTypeName"]

            eng_desc = engine["governanceEngineDescription"]
            eng_req_type = json.dumps(engine["governanceRequestTypes"], indent=2)
            status = engine["governanceEngineStatus"]
            if status in ("RUNNING"):
                eng_status = f"[green]{status}"
            elif status in ("FAILED"):
                eng_status = f"[red]{status}"
            else:
                eng_status = f"[yellow]{status}"

            table.add_row(gov_eng, eng_type, eng_desc, eng_status, eng_req_type)

        table.caption = f"Server {server_name} running on {url}"
        return table

    try:
        if paging is True:
            console = Console(width=width, force_terminal=not jupyter)
            with console.pager():
                console.print(generate_table())
        else:
            with Live(
                generate_table(),
                refresh_per_second=1,
                screen=True,
                vertical_overflow="visible",
            ) as live:
                while True:
                    time.sleep(2)
                    live.update(generate_table())

    except (
        InvalidParameterException,
        PropertyServerException,
        UserNotAuthorizedException,
    ) as e:
        print_exception_response(e)

    except KeyboardInterrupt:
        pass

    finally:
        s_client.close_session()


def main_live():
    parser = argparse.ArgumentParser()
    parser.add_argument("--server", help="Name of the server to display status for")
    parser.add_argument("--url", help="URL Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    parser.add_argument("--password", help="User Password")
    args = parser.parse_args()

    server = args.server if args.server is not None else EGERIA_ENGINE_HOST
    url = args.url if args.url is not None else EGERIA_ENGINE_HOST_URL
    userid = args.userid if args.userid is not None else EGERIA_USER
    user_pass = args.password if args.password is not None else EGERIA_USER_PASSWORD
    display_gov_eng_status(
        server=server, url=url, username=userid, user_pass=user_pass, paging=False
    )


def main_paging():
    parser = argparse.ArgumentParser()
    parser.add_argument("--server", help="Name of the server to display status for")
    parser.add_argument("--url", help="URL Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    parser.add_argument("--password", help="User Password")
    args = parser.parse_args()

    server = args.server if args.server is not None else EGERIA_ENGINE_HOST
    url = args.url if args.url is not None else EGERIA_ENGINE_HOST_URL
    userid = args.userid if args.userid is not None else EGERIA_USER
    user_pass = args.password if args.password is not None else EGERIA_USER_PASSWORD
    display_gov_eng_status(
        server=server, url=url, username=userid, user_pass=user_pass, paging=True
    )


if __name__ == "__main__":
    main_live()

if __name__ == "__main_paging__":
    main_paging()
